/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.lang.reflect.InvocationTargetException;

import javax.sound.sampled.*;
import javax.swing.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Application
{
    private final Logger _logger = LoggerFactory.getLogger(getClass().getCanonicalName());

    public Application(String[] args)
    {
        boolean dumpAudio = false;

        for (String s: args) {
            if (s.equalsIgnoreCase("-a")) {
                dumpAudio = true;
            }
        }

        if (dumpAudio) {
            // We need all these to help us determine capabilities of lines
            Line.Info sourceDLInfo = new Line.Info(SourceDataLine.class);
            Line.Info targetDLInfo = new Line.Info(TargetDataLine.class);

            Mixer.Info mixers[] = AudioSystem.getMixerInfo();
            for (int i=0; i<mixers.length; i++) {
                Mixer thisMixer = AudioSystem.getMixer(mixers[i]);
                if (thisMixer.isLineSupported(sourceDLInfo)) {
                    System.out.println("Output Mixer " + i + ": " + mixers[i].getDescription() + " - " + mixers[i].getName());
                }
            }
            for (int i=0; i<mixers.length; i++) {
                Mixer thisMixer = AudioSystem.getMixer(mixers[i]);
                if (thisMixer.isLineSupported(targetDLInfo)) {
                    System.out.println("Input Mixer " + i + ": " + mixers[i].getDescription() + " - " + mixers[i].getName());
                }
            }
            System.exit(0);
        }

        try
        {
            if (System.getProperty("audio.input") != null) {
                int index = Integer.parseInt(System.getProperty("audio.input"));
                Configuration.getInstance().setIntegerProperty("audio-input", index);
            }
        }
        catch (Exception exception)
        {
            _logger.info("No audio input mixer specified in the configuration. Using default mixer...");
        }
        try
        {
            if (System.getProperty("audio.output") != null) {
                int index = Integer.parseInt(System.getProperty("audio.output"));
                Configuration.getInstance().setIntegerProperty("audio-output", index);
            }
        }
        catch (Exception exception)
        {
            _logger.info("No audio output mixer specified in the configuration. Using default mixer...");
        }

        // Before compiling the code, please get a Zypr API key
        // at http://developer.zypr.net and uncomment/edit the line below
        // System.setProperty("net.zypr.api.key", "REPLACE THIS WITH YOUR API KEY");
        try
        {
            if (System.getProperty("net.zypr.api.url") == null)
                System.setProperty("net.zypr.api.url", Configuration.getInstance().getProperty("api-url"));
        }
        catch (Exception exception)
        {
            _logger.info("No api-url specified in the configuration. Connection to default API URL...");
        }
        try
        {
            UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
        }
        catch (Exception exception)
        {
            _logger.warn(exception.toString());
        }

        SwingUtilities.invokeLater(new Runnable()
        {
            @Override
            public void run() {
                JFrame frame = new ApplicationFrame();
                Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
                Dimension frameSize = frame.getSize();
                if (frameSize.height > screenSize.height)
                    frameSize.height = screenSize.height;
                if (frameSize.width > screenSize.width)
                    frameSize.width = screenSize.width;
                frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
                frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
                frame.setIconImage(Resources.getInstance().getImage("logo-app.png"));
                frame.setVisible(true);
            }
        });
    }

    public static void main(String[] args)
    {
        new Application(args);
    }
}
