/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;
import java.awt.*;
import java.io.*;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.KeySpec;
import java.util.Properties;

public class Branding extends Properties
{
    private final Logger _logger = LoggerFactory.getLogger(getClass().getCanonicalName());
    private static final Branding INSTANCE = new Branding();
    private static final char[] HEX_CHARS =
            {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
    private static final String _propertiesFilePath = "/net/zypr/resources/branding.xml";

    private Branding()
    {
        super();
        load();
    }

    public static synchronized Branding getInstance()
    {
        return (INSTANCE);
    }

    public synchronized void load()
    {
        try {
            final InputStream fileInputStream = getClass().getResourceAsStream(_propertiesFilePath);
            loadFromXML(fileInputStream);
            fileInputStream.close();
            _logger.info("BRANDING LOADED FROM " + _propertiesFilePath);
        } catch (FileNotFoundException fileNotFoundException) {
            _logger.error("", fileNotFoundException);
        } catch (IOException ioException) {
            _logger.error("", ioException);
        }
    }

    public synchronized int getIntegerProperty(String key, int defaultValue)
    {
        int value = defaultValue;
        try {
            value = Integer.parseInt(getProperty(key, "" + defaultValue));
        } catch (NumberFormatException numberFormatException) {
            _logger.warn(numberFormatException.toString());
        }
        return (value);
    }

    public synchronized float getFloatProperty(String key, float defaultValue)
    {
        float value = defaultValue;
        try {
            value = Float.parseFloat(getProperty(key, "" + defaultValue));
        } catch (NumberFormatException numberFormatException) {
            _logger.warn(numberFormatException.toString());
        }
        return (value);
    }

    public synchronized Color getColorProperty(String key, Color defaultValue)
    {
        Color value = defaultValue;
        try {
            String[] values = getProperty(key, "" + value.getRed() + "," + value.getGreen() + "," + value.getBlue() + "," + value.getAlpha()).split(",");
            value = new Color(Integer.parseInt(values[0]), Integer.parseInt(values[1]), Integer.parseInt(values[2]), Integer.parseInt(values[3]));
        } catch (Exception exception) {
            _logger.error("", exception);
        }
        return (value);
    }

    public synchronized boolean getBooleanProperty(String key, boolean defaultValue)
    {
        boolean value = defaultValue;
        String strValue = getProperty(key, "" + defaultValue).trim().toLowerCase();
        if (strValue.equals("true"))
            value = true;
        else if (strValue.equals("false"))
            value = false;
        return (value);
    }

    public synchronized String getStringProperty(String key, String value)
    {
        String path = getProperty(key, value);
        return path;
    }

    private synchronized String bytesToHexString(byte[] byteArray)
    {
        StringBuffer stringBuffer = new StringBuffer(byteArray.length * 2);
        for (int i = 0; i < byteArray.length; i++) {
            stringBuffer.append(HEX_CHARS[(byteArray[i] & 0xf0) >>> 4]);
            stringBuffer.append(HEX_CHARS[byteArray[i] & 0x0f]);
        }
        return (stringBuffer.toString());
    }

    private synchronized byte[] hexStringToBytes(String hexString)
    {
        byte byteArray[] = new byte[hexString.length() / 2];
        for (int index = 0; index < (hexString.length() / 2); index++) {
            byte firstNibble = Byte.parseByte(hexString.substring(2 * index, 2 * index + 1), 16);
            byte secondNibble = Byte.parseByte(hexString.substring(2 * index + 1, 2 * index + 2), 16);
            int finalByte = (secondNibble) | (firstNibble << 4);
            byteArray[index] = (byte) finalByte;
        }
        return (byteArray);
    }

    @Override
    public synchronized String getProperty(String key, String defaultValue)
    {
        String value = super.getProperty(key, defaultValue);
        setProperty(key, value);
        return (value);
    }
}
