/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.zypr.api.Session;
import net.zypr.gui.utils.FileUtils;

public class Settings
{
    private final Logger _logger = LoggerFactory.getLogger(getClass().getCanonicalName());
    private static final Settings INSTANCE = new Settings();
    private static SettingsDTO _dto = null;

    public Settings()
    {
        super();
    }

    public synchronized static Settings getInstance()
    {
        return (INSTANCE);
    }

    public synchronized SettingsDTO getDTO()
    {
        if (_dto == null)
            load();
        return (_dto);
    }

    public synchronized void load()
    {
        File file = new File(FileUtils.ZYPR_PATH, File.separator + "settings" + File.separator);
        XMLDecoder xmlDecoder = null;
        try {
            xmlDecoder = new XMLDecoder(new BufferedInputStream(new FileInputStream(new File(file, "" + Session.getInstance().getUsername().hashCode()))));
            _dto = (SettingsDTO) xmlDecoder.readObject();
            _logger.info("CONFIGURATION LOADED FROM " + (new File(file, "" + Session.getInstance().getUsername().hashCode())));
            if (_dto == null)
                _dto = new SettingsDTO();
        } catch (FileNotFoundException fileNotFoundException) {
            _dto = new SettingsDTO();
            _logger.warn(fileNotFoundException.getMessage());
        } catch (Exception exception) {
            _dto = new SettingsDTO();
            _logger.error("", exception);
        } finally {
            if (xmlDecoder != null)
                xmlDecoder.close();
            xmlDecoder = null;
        }
    }

    public synchronized void save()
    {
        File file = new File(FileUtils.ZYPR_PATH, File.separator + "settings" + File.separator);
        file.mkdirs();
        XMLEncoder xmlEncoder = null;
        try {
            xmlEncoder = new XMLEncoder(new BufferedOutputStream(new FileOutputStream(new File(file, "" + Session.getInstance().getUsername().hashCode()))));
            xmlEncoder.writeObject(_dto);
            _logger.info("SETTINGS SAVED INTO " + (new File(file, "" + Session.getInstance().getUsername().hashCode())));
        } catch (Exception exception) {
            _logger.error("", exception);
        } finally {
            if (xmlEncoder != null)
                xmlEncoder.close();
            xmlEncoder = null;
        }
    }

    public synchronized void clear()
    {
        _dto = null;
    }
}
