/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.components;

import org.jdesktop.swingx.JXPanel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class RoundPanel extends JXPanel
{
    private Timer _timer = null;
    protected final Logger _logger = LoggerFactory.getLogger(getClass().getCanonicalName());

    public RoundPanel(LayoutManager layoutManager, boolean isDoubleBuffered)
    {
        super(layoutManager, isDoubleBuffered);
    }

    public RoundPanel(LayoutManager layoutManager)
    {
        super(layoutManager);
    }

    public RoundPanel(boolean isDoubleBuffered)
    {
        super(isDoubleBuffered);
    }

    public RoundPanel()
    {
        super();
    }

    public void showPanel()
    {
        setEnabledComponents(true);
        if (_timer != null)
        {
            _timer.stop();
            _timer = null;
        }
        this.setAlpha(0f);
        this.setVisible(true);
        _timer = new Timer(50, new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                float alpha = getAlpha() + 0.1f;
                if (alpha > 1.0f)
                {
                    _timer.stop();
                    _timer = null;
                    setAlpha(1.0f);
                }
                else
                {
                    setAlpha(alpha);
                }
            }
        });
        _timer.start();
    }

    public void hidePanel()
    {
        setEnabledComponents(false);
        if (_timer != null)
        {
            _timer.stop();
            _timer = null;
        }
        this.setAlpha(1.0f);
        _timer = new Timer(50, new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                float alpha = getAlpha() - 0.1f;
                if (alpha < 0.0f)
                {
                    _timer.stop();
                    _timer = null;
                    setAlpha(0.0f);
                    setVisible(false);
                }
                else
                {
                    setAlpha(alpha);
                }
            }
        });
        _timer.start();
    }

    public void setEnabledComponents(boolean enabled)
    {
        Component[] components = getComponents();
        for (int index = 0; index < components.length; index++)
            if (components[index] instanceof Button)
                components[index].setEnabled(enabled);
    }

    /** Stroke size. it is recommended to set it to 1 for better view */
    protected int strokeSize = 1;
    /** Color of shadow */
    protected Color shadowColor = Color.black;
    /** Sets if it drops shadow */
    protected boolean shady = false;
    /** Sets if it has an High Quality view */
    protected boolean highQuality = true;
    /** Double values for Horizontal and Vertical radius of corner arcs */
    protected Dimension arcs = new Dimension(20, 20);
    /** Distance between shadow border and opaque panel border */
    protected int shadowGap = 5;
    /** The offset of shadow.  */
    protected int shadowOffset = 4;
    /** The transparency value of shadow. ( 0 - 255) */
    protected int shadowAlpha = 150;

    @Override
    protected void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        int width = getWidth();
        int height = getHeight();
        int shadowGap = this.shadowGap;
        Color shadowColorA = new Color(shadowColor.getRed(),
                shadowColor.getGreen(), shadowColor.getBlue(), shadowAlpha);
        Graphics2D graphics = (Graphics2D) g;

        //Sets antialiasing if HQ.
        if (highQuality) {
            graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
        }

        //Draws shadow borders if any.
        if (shady) {
            graphics.setColor(shadowColorA);
            graphics.fillRoundRect(
                    shadowOffset,// X position
                    shadowOffset,// Y position
                    width - strokeSize - shadowOffset, // width
                    height - strokeSize - shadowOffset, // height
                    arcs.width, arcs.height);// arc Dimension
        } else {
            shadowGap = 1;
        }

        //Draws the rounded opaque panel with borders.
        graphics.setColor(getBackground());
        graphics.fillRoundRect(0, 0, width - shadowGap,
                height - shadowGap, arcs.width, arcs.height);
        //graphics.setColor(getForeground());
        //graphics.setStroke(new BasicStroke(strokeSize));
        //graphics.drawRoundRect(0, 0, width - shadowGap,
//                height - shadowGap, arcs.width, arcs.height);

        //Sets strokes to default, is better.
        graphics.setStroke(new BasicStroke());
    }

}
