/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.panels;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.API;
import net.zypr.api.Session;
import net.zypr.api.enums.TemperatureScale;
import net.zypr.api.vo.WeatherCurrentVO;
import net.zypr.gui.Configuration;
import net.zypr.gui.ImageFetcher;
import net.zypr.gui.Resources;
import net.zypr.gui.Settings;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.EmbossLabel;
import net.zypr.gui.components.Panel;
import net.zypr.gui.utils.AudioUtils;
import net.zypr.gui.utils.Converter;
import net.zypr.gui.utils.ImageUtils;
import net.zypr.gui.windows.WeatherCurrentWindow;
import org.jdesktop.swingx.painter.AlphaPainter;
import org.jdesktop.swingx.painter.MattePainter;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;

public class TopPanel extends GenericPanel implements MouseListener
{
    private MapViewPanel _mvp = null;
    //private Label _labelLocation = new Label();
    private EmbossLabel _labelLocation = new EmbossLabel();
    private Button _buttonFindMe = new Button();
    private Panel _panelFlipCard = new Panel();
    private Button _buttonWeather = new Button();
    private NowPlayingPanel _nowPlayingPanel = new NowPlayingPanel();
    private MessagePanel _messagePanel = new MessagePanel();

    //private Label _labelContacting = new Label();
    // Timer for showing new message on the message panel
    private Timer _timer = null;

    // Timer for press and hold or long mouse click
    private Timer _timerHold = new Timer(2000, new ActionListener()
    {
        public void actionPerformed(ActionEvent actionEvent)
        {
            _timerHold.stop();
            AudioUtils.play("button-down.wav");
        }
    });

    public TopPanel(MapViewPanel mvp)
    {
        _mvp = mvp;
        try {
            jbInit();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        _logger.trace("TopPanel init");

        setSize(0, 60);
        setAlpha(1f);
        setOpaque(false);

        // Make background semi-transparent
        AlphaPainter alphaPainter = new AlphaPainter();
        alphaPainter.setPainters(new MattePainter(new Color(0, 0, 0, 196)));
        setBackgroundPainter(alphaPainter);
        setBackground(new Color(0,0,0,196));

        MigLayout layout = new MigLayout("ins 0, fill", "", "[50]");
        setLayout(layout);
        add(_buttonFindMe, "align center, width 20!, height 20!");
        add(_labelLocation, "align center, width 150::");
        add(_panelFlipCard, "gapleft 10px, grow, push");
        add(_buttonWeather, "align center, height 50!, width 50!");

        _panelFlipCard.setOpaque(false);
        _panelFlipCard.setBackground(new Color(0, 0, 0, 255));

        _buttonWeather.setBackground(new Color(0, 0, 0, 255));
        _buttonWeather.addMouseListener(this);

        BufferedImage image = Resources.getInstance().getBufferedImage("icon_findme_on.png");
        ImageIcon imageIcon = new ImageIcon(ImageUtils.scaleFit(image, 20, 20));
        _buttonFindMe.setBackground(new Color(0, 0, 0, 0));
        _buttonFindMe.setIcons(imageIcon, imageIcon, imageIcon);
        _buttonFindMe.addMouseListener(this);

        _labelLocation.setText("");
        _labelLocation.setHorizontalAlignment(SwingConstants.LEFT);
        _labelLocation.setFont(new Font("Dialog", Font.PLAIN, 20));

        _panelFlipCard.setOpaque(false);
        _panelFlipCard.setLayout(null);
        _panelFlipCard.addComponentListener(new ComponentAdapter()
        {
            public void componentResized(ComponentEvent e)
            {
                _panelFlipCard_componentResized(e);
            }
        });
        _panelFlipCard.add(_nowPlayingPanel, null);
        _panelFlipCard.add(_messagePanel, null);

        // NowPlayingPanel defaults to visible
        _nowPlayingPanel.setVisible(true);
        // MessagePanel only visible when needed
        _messagePanel.setVisible(false);
        revalidate();
        repaint();


        // TODO: use these settings
        //_labelAddress.setForeground(Configuration.getInstance().getColorProperty("location-bar-font-color", Color.WHITE));
        //_labelWeather.setForeground(Configuration.getInstance().getColorProperty("weather-bar-font-color", Color.WHITE));


        /*
        Dimension animatedIconSize = new Dimension(Resources.getInstance().getImageIcon("animated-icon-processing.gif").getIconWidth(), Resources.getInstance().getImageIcon("animated-icon-processing.gif").getIconHeight());
        _labelContacting.setText("");
        _labelContacting.setMinimumSize(animatedIconSize);
        _labelContacting.setMaximumSize(animatedIconSize);
        _labelContacting.setPreferredSize(animatedIconSize);
        _panelFlipCard.setOpaque(false);
        _panelFlipCard.setLayout(null);
        _panelFlipCard.addComponentListener(new ComponentAdapter()
        {
            public void componentResized(ComponentEvent e)
            {
                _panelFlipCard_componentResized(e);
            }
        });
        _nowPlayingPanel.setLocation(0, 0);
        _messagePanel.setLocation(0, 0);
        _messagePanel.setVisible(false);
        _panelFlipCard.add(_nowPlayingPanel, null);
        _panelFlipCard.add(_messagePanel, null);
        add(_labelContacting, BorderLayout.EAST);
        add(_panelFlipCard, BorderLayout.CENTER);
        */
        /*
        Session.getInstance().addPropertyChangeListener(new PropertyChangeListener()
        {
            public void propertyChange(PropertyChangeEvent propertyChangeEvent)
            {
                if (propertyChangeEvent.getPropertyName().equals("ActiveRequestCount")) {
                    if (Session.getInstance().getActiveRequestCount() != 0)
                        _labelContacting.setIcon(Resources.getInstance().getImageIcon("animated-icon-processing.gif"));
                    else
                        _labelContacting.setIcon(null);
                }
            }
        });*/
    }

    private void _panelFlipCard_componentResized(ComponentEvent e)
    {
        _nowPlayingPanel.setSize(_panelFlipCard.getSize());
    }

    public void setNotificationMessage(String message)
    {
        setNotificationMessage(message, true, null);
    }

    public void setNotificationMessage(final String message, boolean ttsEnabled, String sound)
    {
        _messagePanel.setNotificationMessage(message, ttsEnabled, sound);
        if (_timer != null && _timer.isRunning()) {
            _timer.restart();
        } else {
            _nowPlayingPanel.hidePanel();
            _messagePanel.showPanel();
            _timer = new Timer(Configuration.getInstance().getIntegerProperty("notification-bar-timeout", 15000), new ActionListener()
            {
                public void actionPerformed(ActionEvent actionEvent)
                {
                    _timer.stop();
                    _timer = null;
                    _messagePanel.hidePanel();
                    _nowPlayingPanel.showPanel();
                }
            });
            _timer.start();
        }
    }

    public void setLocationName(String name)
    {
        if (name == null || name.equals(""))
            return;
        if (!name.equals(_labelLocation.getText())) {
            _labelLocation.setText(name);
            revalidate();
            updateCurrentWeather();
        }
    }

    private void displayCurrentWeather()
    {
        new Thread(new Runnable()
        {
            WeatherCurrentVO weatherCurrentVO = null;
            public void run()
            {
                try {
                    weatherCurrentVO = API.getInstance().getWeather().current("default", _labelLocation.getText());
                    // TODO: assume that discardAndShowWindow and showWindow are thread safe
                    if (_mvp.getLastInStack() instanceof WeatherCurrentWindow) {
                        _mvp.discardAndShowWindow(new WeatherCurrentWindow("default", weatherCurrentVO, _labelLocation.getText()));
                    } else {
                        _mvp.showWindow(new WeatherCurrentWindow("default", weatherCurrentVO, _labelLocation.getText()));
                    }

                } catch (Exception exception) {
                    _logger.error("", exception);
                }
            }
        }).start();
    }

    public void updateCurrentWeather()
    {
        new Thread(new Runnable()
        {
            ImageIcon _weatherImage = null;
            String weatherText = null;
            public void run()
            {
                try {
                    if (!_labelLocation.getText().trim().equals("") && !_labelLocation.getText().trim().equals("...")) {
                        WeatherCurrentVO weatherCurrentVO = API.getInstance().getWeather().current(_labelLocation.getText());
                        BufferedImage image = ImageFetcher.getInstance().getImage(weatherCurrentVO.getIconURI(), 0, 0, true);
                        _weatherImage = new ImageIcon(ImageUtils.scaleFit(image, 50, 50));

                        TemperatureScale temperatureScale = Settings.getInstance().getDTO().getTemperatureScale();
                        weatherText = ((int) Converter.convertTemperatures(weatherCurrentVO.getScale(), temperatureScale, weatherCurrentVO.getCurrentTemperature())) + " " + temperatureScale.getSymbol() + " - " + weatherCurrentVO.getDescription();
                    } else {
                        weatherText = "";
                    }
                } catch (Exception exception) {
                    _logger.error("", exception);
                }

                // Update on the UI thread
                SwingUtilities.invokeLater(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        if (_weatherImage != null) {
                            _buttonWeather.setIcons(_weatherImage, _weatherImage, _weatherImage);
                            repaint();
                        }
                        // TODO: figure out what we want to do here
                        _logger.debug("weather text - {}", weatherText);
                    }
                });

            }
        }).start();
    }


    @Override
    protected void paintComponent(Graphics graphics)
    {
        super.paintComponent(graphics);
        graphics.setColor(getBackground());
        graphics.fillRect(0, 0, getWidth(), getHeight());
    }

    @Override
    public void mouseClicked(MouseEvent mouseEvent)
    {
    }

    @Override
    public void mousePressed(MouseEvent mouseEvent)
    {
        _timerHold.restart();
    }

    @Override
    public void mouseReleased(MouseEvent mouseEvent)
    {
        if (_timerHold.isRunning()) {
            _timerHold.stop();
            if (mouseEvent.getSource() == _buttonFindMe) {
                _mvp.setMapCenter(Session.getInstance().getPosition());
            } else if (mouseEvent.getSource() == _buttonWeather) {
                displayCurrentWeather();
            }
        } else {
            _timerHold.stop();
            if (mouseEvent.getSource() == _buttonFindMe) {
                _logger.debug("Setting my location to map center");
                _mvp.setLocationToMapCenter();
            }
        }
    }

    @Override
    public void mouseEntered(MouseEvent mouseEvent)
    {
    }

    @Override
    public void mouseExited(MouseEvent mouseEvent)
    {
    }
}
