/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.renderers;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.image.BufferedImage;

import javax.swing.JList;
import javax.swing.ListCellRenderer;

import net.zypr.api.API;
import net.zypr.api.vo.ItemVO;
import net.zypr.api.vo.ServiceVO;
import net.zypr.gui.ImageFetcher;

public class POIListRenderer
        extends POIListRowPanel
        implements ListCellRenderer
{
    public POIListRenderer()
    {
        super();
        setOpaque(false);
        setPreferredSize(new Dimension(40, 47));
    }

    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
    {
        setOpaque(isSelected);
        ItemVO itemVO = (ItemVO) value;
        setLineNumber(index + 1);
        setName(itemVO.getName());
        ServiceVO serviceVO = null;
        BufferedImage bufferedImage = null;
        try
        {
            serviceVO = API.getInstance().getPOI().getService(itemVO.getService());
            bufferedImage = (BufferedImage) serviceVO.getServiceIconObject();
        }
        catch (Exception exception)
        {
            _logger.warn("Unknown service : {}", itemVO.getService());
        }
        if (bufferedImage == null && serviceVO != null)
        {
            bufferedImage = ImageFetcher.getInstance().getImage(serviceVO.getServiceIconURI(), 24, 24);
            if (bufferedImage != null)
                serviceVO.setServiceIconObject(bufferedImage);
        }
        if (bufferedImage != null)
            setServiceIcon(bufferedImage);
        bufferedImage = ImageFetcher.getInstance().getImage(itemVO.getIconURL(), 32, 32);
        if (bufferedImage != null)
            setPicture(bufferedImage);
        return this;
    }
}
