/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.API;
import net.zypr.api.Session;
import net.zypr.api.enums.StatusCode;
import net.zypr.api.exceptions.APICommunicationException;
import net.zypr.api.exceptions.APIProtocolException;
import net.zypr.api.exceptions.APIServerErrorException;
import net.zypr.gui.*;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.CheckBox;
import net.zypr.gui.components.Label;
import net.zypr.gui.components.PasswordField;
import net.zypr.gui.components.Separator;
import net.zypr.gui.components.TextField;
import net.zypr.gui.utils.DeviceID;
import net.zypr.gui.utils.StringUtils;

import javax.swing.*;

public class LoginWindow extends ModalWindow implements FocusListener
{
    private Label _labelEMailAddress = new Label("Email Address");
    private Label _labelPassword = new Label("Password");
    private TextField _textFieldEMailAddress = new TextField(Configuration.getInstance().getProperty("login-email-address", ""));
    private PasswordField _passwordFieldPassword = new PasswordField(Configuration.getInstance().getEncryptedProperty("login-password", ""));
    private Separator _separatorTop = new Separator();
    private Separator _separatorBottom = new Separator();
    private CheckBox _checkBoxRememberMe = new CheckBox("Remember me");
    private CheckBox _checkBoxAutoSignIn = new CheckBox("Automatically log in");
    private Button _buttonSignIn = new Button("button-sign-in-up.png", "button-sign-in-down.png");
    private Button _buttonSignUp = new Button("button-sign-up-up.png", "button-sign-up-down.png");

    public LoginWindow()
    {
        try {
            jbInit();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        this.setSize(600, 370);

        String login = Branding.getInstance().getStringProperty("login-title-text", "Login");
        setTitle(login);

        BufferedImage image = Resources.getInstance().getBufferedImage("logo-service.png");
        setTitleImage(image);

        MigLayout layout = new MigLayout("ins 5, width 580!", "[fill]", "[]");
        _panelContent.setLayout(layout);

        _panelContent.add(_separatorTop, "push 100, wrap");
        _panelContent.add(_labelEMailAddress, "gapleft 25, gapright 25, gaptop 5,wrap");
        _panelContent.add(_textFieldEMailAddress, "gapleft 25, gapright 25, height 40!, wrap");
        _panelContent.add(_labelPassword, "gapleft 25, gapright 25, gaptop 10, wrap");
        _panelContent.add(_passwordFieldPassword, "gapleft 25, gapright 25, height 40!, wrap");
        _panelContent.add(_checkBoxRememberMe, "gapleft 50, gaptop 10, split 2");
        _panelContent.add(_checkBoxAutoSignIn, "gapright 100, wrap");
        _panelContent.add(_separatorBottom, "push 100, gaptop 5, gapbottom 15, wrap");
        _panelContent.add(_buttonSignIn, "gapleft 100, split 2");
        _panelContent.add(_buttonSignUp, "gapright 100, wrap");


        _labelEMailAddress.setFont(new Font("Dialog", Font.PLAIN, 14));
        _labelPassword.setFont(new Font("Dialog", Font.PLAIN, 14));

        _textFieldEMailAddress.setFont(new Font("Dialog", Font.PLAIN, 20));
        _textFieldEMailAddress.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent keyEvent)
            {
                _textFieldEMailAddress_keyPressed(keyEvent);
            }
        });

        _passwordFieldPassword.setFont(new Font("Dialog", Font.PLAIN, 20));
        _passwordFieldPassword.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent keyEvent)
            {
                _passwordFieldPassword_keyPressed(keyEvent);
            }
        });

        boolean rememberMe = Configuration.getInstance().getBooleanProperty("login-remember-my-email", false);
        rememberMe = rememberMe && Configuration.getInstance().getBooleanProperty("login-remember-my-password", false);
        _checkBoxRememberMe.setChecked(rememberMe);
        _checkBoxAutoSignIn.setChecked(Configuration.getInstance().getBooleanProperty("login-sign-me-in-automatically", false));

        _checkBoxRememberMe.setActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                setCheckBoxes();
            }
        });

        _buttonSignIn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonSignIn_actionPerformed(actionEvent);
            }
        });
        _buttonSignUp.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonSignUp_actionPerformed(actionEvent);
            }
        });

        setCheckBoxes();
    }

    private void setCheckBoxes()
    {
        //_checkBoxRememberPassword.setEnabled(_checkBoxRememberEMail.isChecked());
        _checkBoxAutoSignIn.setEnabled(_checkBoxRememberMe.isChecked());
    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent)
    {
        super._buttonWindowClose_actionPerformed(actionEvent);
        System.exit(0);
    }

    private void doServerLogin() {
        // Try to log in on another thread
        new Thread(new Runnable()
        {
            ModalWindow window = null;
            boolean success = true;
            @Override
            public void run()
            {
                SwingUtilities.invokeLater(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        String title = Branding.getInstance().getStringProperty("connecting-title-text", "Zypr Service");
                        ToastWindow toastWindow = new ToastWindow(title, "Establishing Services", 120, null, "logo-login.png");
                        getParentPanel().showWindow(toastWindow);
                    }
                });

                try {
                    if (API.getInstance().getAuth().login(_textFieldEMailAddress.getText(), new String(_passwordFieldPassword.getPassword()), DeviceID.load(_textFieldEMailAddress.getText())) != StatusCode.SUCCESSFUL) {
                        window = new WarningWindow("Unable to sign in", "Unable to sign in. Please make sure you entered your e-mail address and password correct.", 30, "notification-sound-1.wav");
                        success = false;
                    }
                } catch (APIServerErrorException exception) {
                    window = new WarningWindow("Unable to sign in", "Unable to sign in. Please make sure you entered your e-mail address and password correct.", 30, "notification-sound-1.wav");
                    success = false;
                } catch (APIProtocolException protocolException) {
                    _logger.error("", protocolException);
                    success = false;
                    window = new WarningWindow("Protocol error", protocolException.getMessage(), 30, "notification-sound-1.wav");
                } catch (APICommunicationException communicationException) {
                    _logger.error("", communicationException);
                    success = false;
                    window = new WarningWindow("Communication error", "Unable to sign in due to communication error! Please check your internet connection.", 30, "notification-sound-1.wav");
                } catch (Exception exception) {
                    _logger.error("", exception);
                    success = false;
                    window = new WarningWindow("Error", exception.getMessage(), 30, "notification-sound-1.wav");
                }

                SwingUtilities.invokeLater(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        if (!success) {
                            if (window != null)
                                getParentPanel().discardAndShowWindow(window);
                            return;
                        }

                        // Success logging in
                        DeviceID.save(_textFieldEMailAddress.getText(), Session.getInstance().getDeviceID());
                        if (_checkBoxRememberMe.isChecked()) {
                            Configuration.getInstance().setProperty("login-email-address", _textFieldEMailAddress.getText());
                            Configuration.getInstance().setEncryptedProperty("login-password", new String(_passwordFieldPassword.getPassword()));
                        } else {
                            Configuration.getInstance().remove("login-email-address");
                            Configuration.getInstance().remove("login-password");
                        }
                        Configuration.getInstance().setBooleanProperty("login-remember-my-email", _checkBoxRememberMe.isChecked());
                        Configuration.getInstance().setBooleanProperty("login-remember-my-password", _checkBoxRememberMe.isChecked());
                        Configuration.getInstance().setBooleanProperty("login-sign-me-in-automatically", _checkBoxAutoSignIn.isChecked());
                        Configuration.getInstance().save();

                        // Update user info
                        try {
                            Session.getInstance().setUserInfo(API.getInstance().getUser().infoGet());
                        } catch (Exception exception) {
                            _logger.error("", exception);
                        }

                        // Show the mapView
                        getParentPanel().getMainPanel().showMapViewPanel();
                    }
                });

            }
        }).start();
    }
    
    public void _buttonSignIn_actionPerformed(ActionEvent actionEvent)
    {
        // Do some quick validation
        _textFieldEMailAddress.setText(_textFieldEMailAddress.getText().trim());
        if (StringUtils.isEmpty(new String(_passwordFieldPassword.getPassword()))) {
            getParentPanel().showWindow(new WarningWindow("Please enter a password", "Password field is empty. Please enter your password.", 30, "notification-sound-1.wav"));
            return;
        } else if ((new String(_passwordFieldPassword.getPassword())).length() < 4) {
            getParentPanel().showWindow(new WarningWindow("Invalid passwords", "Invalid passwords. Passwords must be at least four characters in length.", 30, "notification-sound-1.wav"));
            return;
        }

        doServerLogin();
    }

    private void _buttonSignUp_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().showWindow(new SignUpWindow());
    }

    private void _textFieldEMailAddress_keyPressed(KeyEvent keyEvent)
    {
        if (keyEvent.getKeyCode() == KeyEvent.VK_ENTER)
            _buttonSignIn_actionPerformed(null);
    }

    private void _passwordFieldPassword_keyPressed(KeyEvent keyEvent)
    {
        if (keyEvent.getKeyCode() == KeyEvent.VK_ENTER)
            _buttonSignIn_actionPerformed(null);
    }

    protected void _this_componentShown(ComponentEvent componentEvent)
    {
        _textFieldEMailAddress.requestFocus();
    }
    @Override
    public void focusGained(FocusEvent focusEvent)
    {
        _logger.debug("focus gained");
    }

    @Override
    public void focusLost(FocusEvent focusEvent)
    {
        _logger.debug("focus lost");
    }

}
