/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.API;
import net.zypr.api.enums.ItemType;
import net.zypr.api.enums.StatusCode;
import net.zypr.api.enums.VoiceCommandAction;
import net.zypr.api.enums.VoiceCommandContextState;
import net.zypr.api.enums.VoiceCommandState;
import net.zypr.api.exceptions.APICommunicationException;
import net.zypr.api.exceptions.APIProtocolException;
import net.zypr.api.exceptions.APIServerErrorException;
import net.zypr.api.vo.*;
import net.zypr.gui.ImageFetcher;
import net.zypr.gui.Settings;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.ImagePanel;
import net.zypr.gui.components.Label;
import net.zypr.gui.components.ScrollPane;
import net.zypr.gui.components.TextPane;
import net.zypr.gui.panels.MapViewPanel;
import net.zypr.mmp.mplayer.TTSPlayer;

import javax.swing.*;

public class MessageSendWindow extends ModalWindow
{
    private ImagePanel _imagePanelContactPicture = new ImagePanel("placeholder-picture-contact-96.png");
    private ImagePanel _imagePanelAttachmentPicture = new ImagePanel("placeholder-picture-attachment-96.png");
    private Button _buttonContactDetails = new Button("button-message-send-contact-details-up.png", "button-message-send-contact-details-down.png");
    private Button _buttonOpenAttachment = new Button("button-message-send-open-attachment-up.png", "button-message-send-open-attachment-down.png");
    private Button _buttonSend = new Button("button-message-send-send-up.png", "button-message-send-send-down.png");
    private Button _buttonCancel = new Button("button-message-send-cancel-up.png", "button-message-send-cancel-down.png");
    private Label _labelMessageTo = new Label();
    private Label _labelMessageAttachment = new Label();
    private ScrollPane _scrollPaneMessageText = new ScrollPane();
    private TextPane _textPaneMessageText = new TextPane();
    private ItemVO _contactItemVO = null;
    private ItemVO _attachmentItemVO = null;
    private String _service = null;
    private boolean _publicMessage = false;

    public MessageSendWindow(ItemVO contactItemVO, ItemVO attachmentItemVO, String service)
    {
        super();
        try {
            _contactItemVO = contactItemVO;
            _attachmentItemVO = attachmentItemVO;
            if (service == null && _contactItemVO != null)
                _service = _contactItemVO.getService();
            else
                _service = service;
            jbInit();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    public MessageSendWindow(ItemVO contactItemVO, ItemVO attachmentItemVO, String service, boolean publicMessage)
    {
        this(contactItemVO, attachmentItemVO, service);
        _publicMessage = publicMessage;
    }

    private void jbInit()
            throws Exception
    {
        _logger.trace("MessageSendWindow init");

        this.setSize(new Dimension(560, 300));

        MigLayout layout = new MigLayout("ins 5, fill");
        _panelContent.setLayout(layout);

        _panelContent.add(_imagePanelContactPicture, "spany 2");
        _panelContent.add(_labelMessageTo, "grow, spanx 2");
        _panelContent.add(_imagePanelAttachmentPicture, "spany 2, wrap");
        _panelContent.add(_buttonOpenAttachment, "");
        _panelContent.add(_labelMessageAttachment, "grow, push, wrap");
        _panelContent.add(_scrollPaneMessageText, "spanx, grow, wrap");
        _panelContent.add(_buttonSend, "center, spanx, split 2");
        _panelContent.add(_buttonCancel, "gapbefore 40px");

        //_panelContent.add(_buttonContactDetails, null);

        _labelMessageAttachment.setLineWrap(true);

        _buttonContactDetails.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonContactDetails_actionPerformed(actionEvent);
            }
        });
        _buttonOpenAttachment.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonOpenAttachment_actionPerformed(actionEvent);
            }
        });
        _buttonSend.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonSend_actionPerformed(actionEvent);
            }
        });
        _buttonCancel.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonCancel_actionPerformed(actionEvent);
            }
        });
        _scrollPaneMessageText.addToViewport(_textPaneMessageText);
        _textPaneMessageText.setEditable(true);

        if (_contactItemVO == null) {
            _imagePanelContactPicture.setVisible(false);
            _buttonContactDetails.setVisible(false);
            _labelMessageTo.setVisible(false);
        }

        if (_attachmentItemVO != null) {
            _labelMessageAttachment.setText("Attachment:" + _attachmentItemVO.getName());
            setTitle("Send Message : Do you want to add text?");
            textToSpeech("Do you want to add text");
            setVoiceCommandState(VoiceCommandState.CONFIRM_ADD_TEXT);
            setVoiceCommandContextState(new VoiceCommandContextState[]
                    {VoiceCommandContextState.ATTACHMENT_AVAILABLE});
        } else {
            _labelMessageAttachment.setText("");
            _buttonOpenAttachment.setVisible(false);
            _imagePanelAttachmentPicture.setVisible(false);
            setTitle("Send Message : Please state message text...");
            textToSpeech("Please state message text");
            setVoiceCommandState(VoiceCommandState.GET_MESSAGE_TEXT);
        }

        new Thread(new Runnable()
        {
            BufferedImage serviceIcon;
            BufferedImage image = null;
            BufferedImage imageAttachment = null;
            public void run()
            {
                try {
                    serviceIcon = ImageFetcher.getInstance().getImage(API.getInstance().getPOI().getService(_service).getServiceIconURI());
                    String iconURL;
                    if (_contactItemVO != null) {
                        iconURL = _contactItemVO.getIconURL();
                        if (iconURL != null && !iconURL.equals("")) {
                            image = ImageFetcher.getInstance().getImage(iconURL, 96, 96, true);
                        }
                    }
                    if (_attachmentItemVO != null) {
                        iconURL = _attachmentItemVO.getIconURL();
                        if (iconURL != null && !iconURL.equals("")) {
                            imageAttachment = ImageFetcher.getInstance().getImage(iconURL, 96, 96, true);
                        }
                    }
                } catch (Exception exception) {
                    _logger.error("", exception);
                }

                SwingUtilities.invokeLater(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        setTitleImage(serviceIcon);
                        if (_contactItemVO != null && image != null) {
                            _labelMessageTo.setText("To : " + _contactItemVO.getName());
                            _imagePanelContactPicture.setImage(image);
                        }
                        if (_attachmentItemVO != null && imageAttachment != null) {
                            _imagePanelAttachmentPicture.setImage(imageAttachment);
                        }
                        revalidate();
                        repaint();
                    }
                });
            }
        }).start();
    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().discardWindow();
    }

    private void _buttonContactDetails_actionPerformed(ActionEvent actionEvent)
    {
        new Thread(new Runnable()
        {
            public void run()
            {
                try {
                    getParentPanel().showWindow(new ContactDetailsWindow(_contactItemVO, (InfoContactTypeVO) API.getInstance().getInfoVO(_contactItemVO.getAction("details").getHandlerURI()), false));
                } catch (Exception exception) {
                    _logger.error("", exception);
                }
            }
        }).start();
    }

    // TODO: fix this
    private void _buttonOpenAttachment_actionPerformed(ActionEvent actionEvent)
    {
        if (_attachmentItemVO != null) {
            if (_attachmentItemVO.getType() == ItemType.USER) {
                _buttonOpenAttachment.setEnabled(false);
                new Thread(new Runnable()
                {
                    public void run()
                    {
                        try {
                            getParentPanel().showWindow(new ContactDetailsWindow(_attachmentItemVO, (InfoContactTypeVO) API.getInstance().getInfoVO(_attachmentItemVO.getAction("details").getHandlerURI()), false));
                        } catch (Exception exception) {
                            _logger.error("", exception);
                        } finally {
                            _buttonOpenAttachment.setEnabled(true);
                        }
                    }
                }).start();
            } else if (_attachmentItemVO.getType() == ItemType.RADIO) {
                getParentPanel().showWindow(new RadioStationDetailsWindow(_attachmentItemVO, false));
            } else if (_attachmentItemVO.getType() == ItemType.POI) {
                getParentPanel().showWindow(new POIDetailsWindow(_attachmentItemVO, false));
            } else {
                getParentPanel().showWindow(new WarningWindow("Unable to open attachment", "Unable to open attachment. Unknown attachment type", 30, "notification-sound-1.wav"));
            }
        }
    }

    private void _buttonCancel_actionPerformed(ActionEvent actionEvent)
    {
        _buttonWindowClose_actionPerformed(actionEvent);
    }

    private void _buttonSend_actionPerformed(ActionEvent actionEvent)
    {
        try {
            _textPaneMessageText.setText(_textPaneMessageText.getText().trim());
            if (_publicMessage) {
                if (API.getInstance().getSocial().messageSendPublic(_service, (_contactItemVO == null ? "" : _contactItemVO.getName()), (_service != null ? _service : _contactItemVO.getService()), (_contactItemVO == null ? "" : _contactItemVO.getServiceItemID()), (_contactItemVO == null ? "public" : "private"), _textPaneMessageText.getText(), (_attachmentItemVO != null ? _attachmentItemVO.getRawJSONString() : null)) == StatusCode.SUCCESSFUL) {
                    ((MapViewPanel) getParentPanel()).showTopBarNotification("Public message sent", true, "notification-sound-4.wav");
                    getParentPanel().discardWindow();
                } else {
                    getParentPanel().showWindow(new WarningWindow("Unable to send public message", "Unable to send public message. Please try again later.", 30, "notification-sound-1.wav"));
                }
            } else {
                if (API.getInstance().getSocial().messageSend(_service, (_contactItemVO == null ? "" : _contactItemVO.getName()), (_service != null ? _service : _contactItemVO.getService()), (_contactItemVO == null ? "" : _contactItemVO.getServiceItemID()), (_contactItemVO == null ? "public" : "private"), _textPaneMessageText.getText(), (_attachmentItemVO != null ? _attachmentItemVO.getRawJSONString() : null)) == StatusCode.SUCCESSFUL) {
                    ((MapViewPanel) getParentPanel()).showTopBarNotification("Private message sent", true, "notification-sound-4.wav");
                    getParentPanel().discardWindow();
                } else {
                    getParentPanel().showWindow(new WarningWindow("Unable to send private message", "Unable to send private message. Please try again later.", 30, "notification-sound-1.wav"));
                }
            }
        } catch (APIProtocolException protocolException) {
            _logger.error("", protocolException);
            getParentPanel().showWindow(new WarningWindow("Protocol error", protocolException.getMessage(), 30, "notification-sound-1.wav"));
        } catch (APICommunicationException communicationException) {
            _logger.error("", communicationException);
            getParentPanel().showWindow(new WarningWindow("Communication error", "Unable to send message due to communication error! Please check your internet connection.", 30, "notification-sound-1.wav"));
        } catch (APIServerErrorException serverErrorException) {
            _logger.error("", serverErrorException);
            getParentPanel().showWindow(new WarningWindow("Server error", "Unable to send message due to a protocol error! Please contact to admin.", 30, "notification-sound-1.wav"));
        }
    }

    public boolean processVoiceCommand(VoiceCommandVO voiceCommandVO)
    {
        try {
            if (voiceCommandVO.getName() == VoiceCommandAction.CONFIRMATION_NO && getVoiceCommandState() == VoiceCommandState.CONFIRM_ADD_TEXT) {
                _buttonSend_actionPerformed(null);
            } else if (voiceCommandVO.getName() == VoiceCommandAction.CONFIRMATION_YES && getVoiceCommandState() == VoiceCommandState.CONFIRM_ADD_TEXT) {
                setTitle("Send Message : Please state message text...");
                textToSpeech("Please state message text");
                setVoiceCommandState(VoiceCommandState.GET_MESSAGE_TEXT);
            } else if (voiceCommandVO.getName() == VoiceCommandAction.GET_TEXT) {
                VoiceCommandVariableVO voiceCommandVariableVO = voiceCommandVO.getVariable("user_text");
                _textPaneMessageText.setText(voiceCommandVariableVO.getValueAsString());
                setTitle("Send Message : Is this correct?");
                textToSpeech("Is this correct?");
                setVoiceCommandState(VoiceCommandState.CONFIRM_MESSAGE);
            } else if (voiceCommandVO.getName() == VoiceCommandAction.CONFIRMATION_YES && getVoiceCommandState() == VoiceCommandState.CONFIRM_MESSAGE) {
                _buttonSend_actionPerformed(null);
            } else if (voiceCommandVO.getName() == VoiceCommandAction.CONFIRMATION_NO && getVoiceCommandState() == VoiceCommandState.CONFIRM_MESSAGE) {
                setTitle("Send Message : Please state message text...");
                textToSpeech("Please state message text");
                setVoiceCommandState(VoiceCommandState.GET_MESSAGE_TEXT);
                _textPaneMessageText.setText("");
            } else if (voiceCommandVO.getName() == VoiceCommandAction.SHOW_ATTACHMENT) {
                _buttonOpenAttachment_actionPerformed(null);
            } else {
                return (false);
            }
            return (true);
        } catch (Exception exception) {
            _logger.error("", exception);
        }
        return (false);
    }

    private void textToSpeech(final String text)
    {
        if (Settings.getInstance().getDTO().isTTSCardHeader())
            new Thread(new Runnable()
            {
                public void run()
                {
                    try {
                        TTSPlayer.getInstance().loadPlayfile(API.getInstance().getVoice().tts(text).getAudioURI());
                    } catch (Exception exception) {
                        _logger.error("", exception);
                    }
                }
            }).start();
    }
}
