/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.enums.VoiceCommandContextState;
import net.zypr.api.enums.VoiceCommandState;
import net.zypr.api.vo.ContextDataVO;
import net.zypr.api.vo.VoiceCommandVO;
import net.zypr.gui.ImageFetcher;
import net.zypr.gui.Resources;
import net.zypr.gui.components.*;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.Label;
import net.zypr.gui.components.Panel;
import net.zypr.gui.panels.GenericPanel;
import net.zypr.gui.utils.ImageUtils;
import net.zypr.gui.utils.WebBrowser;
import org.jdesktop.swingx.painter.AlphaPainter;
import org.jdesktop.swingx.painter.MattePainter;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

public class ModalWindow extends Panel
{
    // Image Panel for title bar area
    private ImagePanel _titleImage = new ImagePanel();
    private String _titleImageURL = null;
    private Label _titleLabel = new Label();
    private final Button _buttonWindowClose = new Button("button-close-window-up.png", "button-close-window-down.png");

    // Main Content Panel
    private final RoundPanel _panelTitleBar = new RoundPanel();
    protected final Panel _panelContent = new Panel();



    private Timer _timer = null;
    private ModalWindow _discardWhenHide = null;
    private VoiceCommandState _voiceCommandState = VoiceCommandState.MAIN;
    private VoiceCommandContextState[] _voiceCommandContextStates = null;
    private ArrayList<ContextDataVO> _contextData = new ArrayList<ContextDataVO>();
    public static final int JLIST_VISIBLE_ROW_COUNT = 5;

    public ModalWindow()
    {
        super(true);
        try {
            jbInit();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        MigLayout layout = new MigLayout("ins 5", "[fill,grow]", "");
        this.setLayout(layout);

        // Make background semi-transparent
        AlphaPainter alphaPainter = new AlphaPainter();
        alphaPainter.setPainters(new MattePainter(new Color(0, 0, 0, 196)));
        setBackgroundPainter(alphaPainter);
        setBackground(new Color(0,0,0,196));

        setAlpha(0f);
        setVisible(false);
        setOpaque(false);

        this.add(_panelTitleBar, "wrap");
        this.add(_panelContent, "");

        // Create the title bar panel
        layout = new MigLayout("ins 1");
        _panelTitleBar.setLayout(layout);
        _panelTitleBar.setBackground(new Color(0,0,0,255));
        _panelTitleBar.add(_titleImage, "gapleft 5, width 50!, height 50!");
        _panelTitleBar.add(_titleLabel, "push, grow, gapleft 10px");
        _panelTitleBar.add(_buttonWindowClose, "width 32!, gapright 5");

        // We have (1) title bar area and (2) content area
        _panelContent.setLayout(null);
        _panelContent.setOpaque(false);

        // Title bar area has 2-3 components
        // (1) Image (2) Text Area and possibly (3) close button
        Color bg = new Color(0,0,0,0);
        _titleImage.setBackground(bg);
        BufferedImage image = Resources.getInstance().getBufferedImage("placeholder-picture-unknown-96.png");
        //image = ImageUtils.scaleFit(image, 80, 80);
        _titleImage.setImage(image);

        _titleLabel.setBackground(bg);
        _titleLabel.setText("...");
        _titleLabel.setHorizontalAlignment(SwingConstants.LEFT);
        _titleLabel.setFont(new Font("Dialog", Font.PLAIN, 16));
        _titleLabel.setLineWrap(true);

        _buttonWindowClose.setMaximumSize(new Dimension(32, 32));
        _buttonWindowClose.setMinimumSize(new Dimension(32, 32));
        _buttonWindowClose.setPreferredSize(new Dimension(32, 32));
        _buttonWindowClose.setSize(new Dimension(32, 32));
        _buttonWindowClose.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonWindowClose_actionPerformed(actionEvent);
            }
        });

        addComponentListener(new ComponentAdapter()
        {
            public void componentShown(ComponentEvent componentEvent)
            {
                _this_componentShown(componentEvent);
            }
        });
    }

    public void setParentSize(Dimension parentDimension)
    {
        setLocation((int) (parentDimension.getWidth() - getWidth()) / 2, (int) (parentDimension.getHeight() - getHeight()) / 2);
    }

    public void disableButtonClose() {
        _buttonWindowClose.setVisible(false);
    }

    public void showWindow()
    {
        setEnabledComponents(true);
        if (_timer != null) {
            _timer.stop();
            _timer = null;
        }
        this.setAlpha(0f);
        this.setVisible(true);
        _timer = new Timer(50, new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                float alpha = getAlpha() + 0.15f;
                if (alpha > 1.0f) {
                    _timer.stop();
                    _timer = null;
                    setAlpha(1.0f);
                    windowVisible();
                } else {
                    setAlpha(alpha);
                }
            }
        });
        _timer.start();
    }

    public void hideWindow()
    {
        hideWindow(false);
    }

    public void hideWindow(boolean discard)
    {
        setEnabledComponents(false);
        if (_timer != null) {
            _timer.stop();
            _timer = null;
        }
        this.setAlpha(1.0f);
        if (discard)
            _discardWhenHide = this;
        _timer = new Timer(50, new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                float alpha = getAlpha() - 0.15f;
                if (alpha < 0.0f) {
                    _timer.stop();
                    _timer = null;
                    setAlpha(0.0f);
                    setVisible(false);
                    if (_discardWhenHide != null && getParentPanel() != null)
                        getParentPanel().remove(_discardWhenHide);
                    windowInvisible();
                } else {
                    setAlpha(alpha);
                }
            }
        });
        _timer.start();
    }

    protected void windowVisible()
    {
    }

    protected void windowInvisible()
    {
    }

    public void setEnabledComponents(boolean enabled)
    {
        Component[] components = _panelContent.getComponents();
        for (int index = 0; index < components.length; index++)
            if (components[index] instanceof Button)
                components[index].setEnabled(enabled);
        components = _panelTitleBar.getComponents();
        for (int index = 0; index < components.length; index++)
            if (components[index] instanceof Button)
                components[index].setEnabled(enabled);
    }

    public String getTitleImageURL()
    {
        return (_titleImageURL);
    }

    public void setTitleImageURL(String dataSourceURL)
    {
        _titleImageURL = dataSourceURL;
    }

    public void setTitle(String title)
    {
        _titleLabel.setText(title);
    }

    public String getTitle()
    {
        return (_titleLabel.getText());
    }

    public void setTitleImage(BufferedImage bufferedImage)
    {
        if (bufferedImage != null) {
            _titleImage.setImage(bufferedImage);
        }
    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent)
    {
        hideWindow(true);
    }

    protected void _this_componentShown(ComponentEvent componentEvent)
    {
    }

    protected void _buttonDataSource_actionPerformed(ActionEvent actionEvent)
    {
        if (getTitleImageURL() != null)
            WebBrowser.openURL(getTitleImageURL());
    }

    protected void addWindowCloseButtonActionListener(ActionListener actionListener)
    {
        _buttonWindowClose.addActionListener(actionListener);
    }

    protected void setWindowCloseButtonEnabled(boolean enabled)
    {
        _buttonWindowClose.setEnabled(enabled);
    }

    protected GenericPanel getParentPanel()
    {
        try {
            return ((GenericPanel) getParent());
        } catch (Exception exception) {
            _logger.error("", exception);
            return (null);
        }
    }

    public void setVoiceCommandState(VoiceCommandState voiceCommandState)
    {
        this._voiceCommandState = voiceCommandState;
    }

    public VoiceCommandState getVoiceCommandState()
    {
        return (_voiceCommandState);
    }

    public void setContextData(ArrayList<ContextDataVO> contextData)
    {
        this._contextData = contextData;
    }

    public ArrayList<ContextDataVO> getContextData()
    {
        return (_contextData);
    }

    public void clearContextData()
    {
        _contextData.clear();
    }

    public void addContextData(ContextDataVO contextDataVO)
    {
        _contextData.add(contextDataVO);
    }

    public void setVoiceCommandContextState(VoiceCommandContextState[] voiceCommandContextStates)
    {
        this._voiceCommandContextStates = voiceCommandContextStates;
    }

    public VoiceCommandContextState[] getVoiceCommandContextState()
    {
        return (_voiceCommandContextStates);
    }

    public boolean processVoiceCommand(VoiceCommandVO voiceCommandVO)
    {
        return (false);
    }

    /** Stroke size. it is recommended to set it to 1 for better view */
    protected int strokeSize = 1;
    /** Color of shadow */
    protected Color shadowColor = Color.black;
    /** Sets if it drops shadow */
    protected boolean shady = false;
    /** Sets if it has an High Quality view */
    protected boolean highQuality = true;
    /** Double values for Horizontal and Vertical radius of corner arcs */
    protected Dimension arcs = new Dimension(20, 20);
    /** Distance between shadow border and opaque panel border */
    protected int shadowGap = 5;
    /** The offset of shadow.  */
    protected int shadowOffset = 4;
    /** The transparency value of shadow. ( 0 - 255) */
    protected int shadowAlpha = 150;

    @Override
    protected void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        int width = getWidth();
        int height = getHeight();
        int shadowGap = this.shadowGap;
        Color shadowColorA = new Color(shadowColor.getRed(),
                shadowColor.getGreen(), shadowColor.getBlue(), shadowAlpha);
        Graphics2D graphics = (Graphics2D) g;

        //Sets antialiasing if HQ.
        if (highQuality) {
            graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
        }

        //Draws shadow borders if any.
        if (shady) {
            graphics.setColor(shadowColorA);
            graphics.fillRoundRect(
                    shadowOffset,// X position
                    shadowOffset,// Y position
                    width - strokeSize - shadowOffset, // width
                    height - strokeSize - shadowOffset, // height
                    arcs.width, arcs.height);// arc Dimension
        } else {
            shadowGap = 1;
        }

        //Draws the rounded opaque panel with borders.
        graphics.setColor(getBackground());
        graphics.fillRoundRect(0, 0, width - shadowGap,
                height - shadowGap, arcs.width, arcs.height);
        graphics.setColor(getForeground());
        graphics.setStroke(new BasicStroke(strokeSize));
        graphics.drawRoundRect(0, 0, width - shadowGap,
                height - shadowGap, arcs.width, arcs.height);

        //Sets strokes to default, is better.
        graphics.setStroke(new BasicStroke());
    }
}
