/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;

import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.API;
import net.zypr.api.Session;
import net.zypr.api.enums.ItemDisplay;
import net.zypr.api.enums.MessageType;
import net.zypr.api.vo.InfoPOITypeVO;
import net.zypr.api.vo.ItemVO;
import net.zypr.api.vo.ReviewVO;
import net.zypr.gui.ImageFetcher;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.HTMLPane;
import net.zypr.gui.components.ImagePanel;
import net.zypr.gui.components.Label;
import net.zypr.gui.components.ScrollPane;
import net.zypr.gui.panels.MapViewPanel;
import net.zypr.gui.utils.NumberUtils;
import net.zypr.gui.utils.WebBrowser;

import org.apache.commons.codec.binary.Base64;

public class POIDetailsWindow extends ModalWindow
{
    private ImagePanel _imagePanelPicture = new ImagePanel("placeholder-picture-poi-96.png");
    private ImagePanel _imagePanelRatingIcon = new ImagePanel();
    private Label _labelRatingCount = new Label();
    private Label _labelAddress = new Label();
    private Label _labelPhoneNumber = new Label();

    private ScrollPane _scrollPanelReviews = new ScrollPane();
    private HTMLPane _htmlPaneReviews = new HTMLPane();

    private Button _buttonSendToContact = new Button("button-poi-send-to-contact-up.png", "button-poi-send-to-contact-down.png", "button-poi-send-to-contact-disabled.png");
    private Button _buttonLocateOnMap = new Button("button-poi-locate-on-the-map-up.png", "button-poi-locate-on-the-map-down.png", "button-poi-locate-on-the-map-disabled.png");
    private Button _buttonAddRemoveToRoute = new Button("button-poi-add-to-route-up.png", "button-poi-add-to-route-down.png", "button-poi-add-to-route-disabled.png");
    private Button _buttonAddRemoveToFavorites = new Button("button-poi-add-to-favorite-up.png", "button-poi-add-to-favorite-down.png");

    private boolean _allowSendMessage = true;
    private ItemVO _itemVO = null;

    /*-
    private Button _buttonMakeCall = new Button("button-poi-make-call-up.png", "button-poi-make-call-down.png");
    private Button _buttonAddToCalendar = new Button("button-poi-add-to-calendar-up.png", "button-poi-add-to-calendar-down.png");
    -*/

    public POIDetailsWindow(ItemVO itemVO, boolean allowSendMessage)
    {
        super();
        try {
            _allowSendMessage = allowSendMessage;
            _itemVO = itemVO;
            jbInit();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        // TODO: decide on final height of window
        this.setSize(new Dimension(560, 340));

        setTitle(_itemVO.getName());

        MigLayout layout = new MigLayout("ins 5, fill");
        _panelContent.setLayout(layout);
        _panelContent.add(_imagePanelPicture, " width 96!, height 96!, spany 3");
        _panelContent.add(_imagePanelRatingIcon, "width 84!, height 17!");
        _panelContent.add(_labelRatingCount, "gapleft 10px, grow, wrap");
        _panelContent.add(_labelAddress, "width 400!, spanx 2, wrap");
        _panelContent.add(_labelPhoneNumber, "width 400!, spanx 2, wrap");
        _panelContent.add(_scrollPanelReviews, "gapbottom 10px, grow, spanx, wrap");
        _panelContent.add(_buttonSendToContact, "align center, split 4, spanx");
        _panelContent.add(_buttonLocateOnMap, "gapleft 30px");
        _panelContent.add(_buttonAddRemoveToRoute, "gapleft 30px");
        _panelContent.add(_buttonAddRemoveToFavorites, "gapleft 30px");

        // TODO: decide if make call button should be added

        _scrollPanelReviews.addScrollDownButtonActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _scrollPanelReviews.scrollDown(_scrollPanelReviews.getHeight() / 4);
            }
        });
        _scrollPanelReviews.addScrollUpButtonActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _scrollPanelReviews.scrollUp(_scrollPanelReviews.getHeight() / 4);
            }
        });
        _htmlPaneReviews.addHyperlinkListener(new HyperlinkListener()
        {
            public void hyperlinkUpdate(HyperlinkEvent hyperlinkEvent)
            {
                _htmlPaneReviews_hyperlinkUpdate(hyperlinkEvent);
            }
        });
        _scrollPanelReviews.addToViewport(_htmlPaneReviews);


        _buttonSendToContact.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonSendToContact_actionPerformed(actionEvent);
            }
        });
        _buttonLocateOnMap.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonLocateOnMap_actionPerformed(actionEvent);
            }
        });
        _buttonAddRemoveToRoute.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonAddRemoveToRoute_actionPerformed(actionEvent);
            }
        });
        _buttonAddRemoveToFavorites.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonAddRemoveToFavorites_actionPerformed(actionEvent);
            }
        });

        /*
        _buttonMakeCall.setLocation(270, 70);
        _buttonMakeCall.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonMakeCall_actionPerformed(actionEvent);
            }
        });
        _buttonAddToCalendar.setLocation(410, 70);
        _buttonAddToCalendar.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent actionEvent)
          {
            _buttonAddToCalendar_actionPerformed(actionEvent);
          }
        });
        */

        if (Session.getInstance().isInFavorites(_itemVO))
            _buttonAddRemoveToFavorites.setIcons("button-poi-remove-from-favorite-up.png", "button-poi-remove-from-favorite-down.png", "button-poi-remove-from-favorite-disabled.png");
        else
            _buttonAddRemoveToFavorites.setIcons("button-poi-add-to-favorite-up.png", "button-poi-add-to-favorite-down.png", "button-poi-add-to-favorite-disabled.png");

        new Thread(new Runnable()
        {
            ItemVO detailsItemVO = null;
            BufferedImage serviceIcon;
            BufferedImage image = null;
            BufferedImage ratingImg = null;
            public void run()
            {
                try {
                    // Do the "details" callback
                    API.getInstance().doCallback(_itemVO, "details");

                    detailsItemVO = API.getInstance().getItemVO(_itemVO.getAction("details").getHandlerURI());
                    serviceIcon = ImageFetcher.getInstance().getImage(API.getInstance().getPOI().getService(_itemVO.getService()).getServiceIconURI());
                    String iconURL = detailsItemVO.getIconURL();
                    if (iconURL != null && !iconURL.equals("")) {
                        image = ImageFetcher.getInstance().getImage(iconURL, 0, 0, true);
                    }
                    iconURL = _itemVO.getRatingIconURL() ;
                    if (iconURL != null && !iconURL.equals("")) {
                        ratingImg = ImageFetcher.getInstance().getImage(iconURL, 0, 0, true);
                    }
                } catch (Exception exception) {
                    _logger.error("", exception);
                }

                SwingUtilities.invokeLater(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        setTitleImage(serviceIcon);
                        if (image != null)
                            _imagePanelPicture.setImage(image);

                        _labelRatingCount.setFont(new Font("Dialog",Font.ITALIC,12));
                        if (_itemVO.getRating() <= 0 || ratingImg == null) {
                            _imagePanelRatingIcon.setVisible(false);
                            _labelRatingCount.setVisible(false);
                        } else {
                            int count = _itemVO.getRatingCount();
                            if (count == 1)
                                _labelRatingCount.setText("1 review");
                            else if (count > 1 )
                                _labelRatingCount.setText(count + " reviews");
                        }

                        _imagePanelRatingIcon.setImage(ratingImg);

                        try {
                            InfoPOITypeVO infoPOITypeVO = (InfoPOITypeVO) detailsItemVO.getInfo();
                            if (infoPOITypeVO.getPhoneNumbers() != null)
                                for (int index = 0; index < infoPOITypeVO.getPhoneNumbers().length; index++)
                                    _labelPhoneNumber.setText((index >= 1 ? ", " : "") + infoPOITypeVO.getPhoneNumbers()[index].getValue());
                            else
                                _labelPhoneNumber.setText("");
                            _labelAddress.setText(infoPOITypeVO.getAddress().toAddressString());
                            _labelAddress.setLineWrap(true);
                            setDescription(detailsItemVO.getDescription(), infoPOITypeVO.getReviews());
                        } catch (Exception exception) {
                            _logger.error("", exception);
                        }
                        revalidate();
                    }
                });
            }
        }).start();
    }

    protected void windowVisible()
    {
        _buttonLocateOnMap.setEnabled(_itemVO.getDisplay() == ItemDisplay.GEO);
        _buttonAddRemoveToRoute.setEnabled(_itemVO.getDisplay() == ItemDisplay.GEO);
        _buttonSendToContact.setEnabled(_allowSendMessage);
        MapViewPanel mapViewPanel = (MapViewPanel) getParentPanel();
        mapViewPanel.addItemVOsPOI(_itemVO);
    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().discardWindow();
    }

    private void _buttonSendToContact_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().showWindow(new SendToContactListWindow(_itemVO, MessageType.TEXT));
    }

    private void _buttonLocateOnMap_actionPerformed(ActionEvent actionEvent)
    {
        ((MapViewPanel) getParentPanel()).setMapCenter(_itemVO.getPosition());
        getParentPanel().discardAllWindows();
    }

    private void _buttonAddRemoveToRoute_actionPerformed(ActionEvent actionEvent)
    {
        _buttonAddRemoveToRoute.setEnabled(false);
        ((MapViewPanel) getParentPanel()).addRoute(_itemVO);
        API.getInstance().doCallback(_itemVO, "direction");
    }

    private void _buttonAddRemoveToFavorites_actionPerformed(ActionEvent actionEvent)
    {
        if (Session.getInstance().isInFavorites(_itemVO)) {
            try {
                API.getInstance().getUser().favoriteDelete(_itemVO.getGlobalItemID());
                _buttonAddRemoveToFavorites.setIcons("button-poi-add-to-favorite-up.png", "button-poi-add-to-favorite-down.png", "button-poi-add-to-favorite-disabled.png");
            } catch (Exception exception) {
                _logger.error("", exception);
            }
        } else {
            try {
                API.getInstance().getUser().favoriteSet(_itemVO);
                _buttonAddRemoveToFavorites.setIcons("button-poi-remove-from-favorite-up.png", "button-poi-remove-from-favorite-down.png", "button-poi-remove-from-favorite-disabled.png");
            } catch (Exception exception) {
                _logger.error("", exception);
            }
        }
    }

    /*
    private void _buttonMakeCall_actionPerformed(ActionEvent actionEvent)
    {
        WebBrowser.openURL("data:text/html;base64," + (new Base64()).encodeBase64URLSafeString("<h1>Dialing 123...</h1>".getBytes()));
        API.getInstance().doCallback(_itemVO, "call");
    }

    private void _buttonAddToCalendar_actionPerformed(ActionEvent actionEvent)
    {
    }
    */

    private void _htmlPaneReviews_hyperlinkUpdate(HyperlinkEvent hyperlinkEvent)
    {
        try {
            if (hyperlinkEvent.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                WebBrowser.openURL(hyperlinkEvent.getURL().toString());
                API.getInstance().doCallback(_itemVO, "click");
            }
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void setDescription(String description, ReviewVO[] reviewVOs)
    {
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append("<html><body alink=\"#FF0000\" link=\"#00FF00\" text=\"#FFFFFF\" vlink=\"#0000FF\">");
        if (!description.trim().equals("")) {
            stringBuffer.append("<p>");
            stringBuffer.append(description);
            stringBuffer.append("</p>");
        }
        if (reviewVOs != null)
            for (int index = 0; index < reviewVOs.length; index++) {
                stringBuffer.append("<p>");
                stringBuffer.append("<a href=\"");
                stringBuffer.append(reviewVOs[index].getSourceURI());
                stringBuffer.append("\"/>");
                stringBuffer.append("<img src=\"");
                stringBuffer.append(reviewVOs[index].getReviewerIconURI());
                stringBuffer.append("\" border=\"0\" align=\"left\" width=\"48\"/></a>&nbsp;<b>");
                stringBuffer.append(reviewVOs[index].getReviewerName());
                stringBuffer.append("</b> : ");
                stringBuffer.append(reviewVOs[index].getDescription());
                stringBuffer.append(" (&nbsp;<a style=\"color: #3BB9FF;\" href=\"");
                stringBuffer.append(reviewVOs[index].getSourceURI());
                stringBuffer.append("\"/>More</a>&nbsp;)");
                stringBuffer.append("</p>");
            }
        stringBuffer.append("</body></html>");
        _htmlPaneReviews.setText(stringBuffer.toString());
        _scrollPanelReviews.scrollToStart();
    }
}
