/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.API;
import net.zypr.api.enums.ItemType;
import net.zypr.api.enums.StatusCode;
import net.zypr.api.enums.VoiceCommandAction;
import net.zypr.api.enums.VoiceCommandContextState;
import net.zypr.api.enums.VoiceCommandState;
import net.zypr.api.exceptions.APICommunicationException;
import net.zypr.api.exceptions.APIProtocolException;
import net.zypr.api.exceptions.APIServerErrorException;
import net.zypr.api.vo.InfoContactTypeVO;
import net.zypr.api.vo.ItemVO;
import net.zypr.api.vo.VoiceCommandVO;
import net.zypr.api.vo.VoiceCommandVariableVO;
import net.zypr.gui.ImageFetcher;
import net.zypr.gui.Settings;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.ScrollPane;
import net.zypr.gui.components.TextPane;
import net.zypr.gui.panels.MapViewPanel;
import net.zypr.mmp.mplayer.TTSPlayer;

import javax.swing.*;

public class PostStatusWindow extends ModalWindow
{
    private Button _buttonOpenAttachment = new Button("button-message-send-open-attachment-up.png", "button-message-send-open-attachment-down.png");
    private Button _buttonSend = new Button("button-message-send-send-up.png", "button-message-send-send-down.png");
    private Button _buttonCancel = new Button("button-message-send-cancel-up.png", "button-message-send-cancel-down.png");
    private ScrollPane _scrollPaneMessageText = new ScrollPane();
    private TextPane _textPaneMessageText = new TextPane();
    private ItemVO _attachmentItemVO = null;
    private String _service = null;

    public PostStatusWindow(ItemVO attachmentItemVO, String service)
    {
        super();
        try {
            _attachmentItemVO = attachmentItemVO;
            _service = service;
            jbInit();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        _logger.trace("PostStatusWindow init");

        this.setSize(new Dimension(560, 200));

        MigLayout layout = new MigLayout("ins 5, fill");
        _panelContent.setLayout(layout);
        _panelContent.add(_scrollPaneMessageText, "spanx, grow, wrap");
        _panelContent.add(_buttonOpenAttachment, "center, grow");
        _panelContent.add(_buttonSend, "center, grow");
        _panelContent.add(_buttonCancel, "center, grow");

        int adjustmentPixel = 0;
        if (_attachmentItemVO != null) {
            setVoiceCommandContextState(new VoiceCommandContextState[]
                    {VoiceCommandContextState.ATTACHMENT_AVAILABLE});
            adjustmentPixel = 20;
        } else {
            _buttonOpenAttachment.setVisible(false);
        }
        _buttonOpenAttachment.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonOpenAttachment_actionPerformed(actionEvent);
            }
        });
        _buttonSend.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonSend_actionPerformed(actionEvent);
            }
        });
        _buttonCancel.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonCancel_actionPerformed(actionEvent);
            }
        });
        _scrollPaneMessageText.addToViewport(_textPaneMessageText);
        _textPaneMessageText.setEditable(true);
        setTitle("What is on your mind?");
        textToSpeech("What is on your mind?");
        setVoiceCommandState(VoiceCommandState.GET_MESSAGE_TEXT);

        new Thread(new Runnable()
        {
            BufferedImage serviceIcon = null;
            public void run()
            {
                try {
                    serviceIcon = ImageFetcher.getInstance().getImage(API.getInstance().getSocial().getService(_service).getServiceIconURI());
                } catch (Exception exception) {
                    _logger.error("", exception);
                }

                SwingUtilities.invokeLater(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        if (serviceIcon != null) {
                            setTitleImage(serviceIcon);
                        }
                        revalidate();
                        repaint();
                    }
                });
            }
        }).start();


    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().discardWindow();
    }

    private void _buttonOpenAttachment_actionPerformed(ActionEvent actionEvent)
    {
        if (_attachmentItemVO != null) {
            if (_attachmentItemVO.getType() == ItemType.USER) {
                _buttonOpenAttachment.setEnabled(false);
                new Thread(new Runnable()
                {
                    public void run()
                    {
                        try {
                            getParentPanel().showWindow(new ContactDetailsWindow(_attachmentItemVO, (InfoContactTypeVO) API.getInstance().getInfoVO(_attachmentItemVO.getAction("details").getHandlerURI()), false));
                        } catch (Exception exception) {
                            _logger.error("", exception);
                        } finally {
                            _buttonOpenAttachment.setEnabled(true);
                        }
                    }
                }).start();
            } else if (_attachmentItemVO.getType() == ItemType.RADIO) {
                getParentPanel().showWindow(new RadioStationDetailsWindow(_attachmentItemVO, false));
            } else if (_attachmentItemVO.getType() == ItemType.POI) {
                getParentPanel().showWindow(new POIDetailsWindow(_attachmentItemVO, false));
            } else {
                getParentPanel().showWindow(new WarningWindow("Unable to open attachment", "Unable to open attachment. Unknown attachment type", 30, "notification-sound-1.wav"));
            }
        }
    }

    private void _buttonCancel_actionPerformed(ActionEvent actionEvent)
    {
        _buttonWindowClose_actionPerformed(actionEvent);
    }

    private void _buttonSend_actionPerformed(ActionEvent actionEvent)
    {
        _textPaneMessageText.setText(_textPaneMessageText.getText().trim());
        try {
            if (API.getInstance().getSocial().postStatus(_service, _textPaneMessageText.getText(), (_attachmentItemVO != null ? _attachmentItemVO.getRawJSONString() : null)) == StatusCode.SUCCESSFUL) {
                ((MapViewPanel) getParentPanel()).showTopBarNotification("Your status updated", true, "notification-sound-4.wav");
                getParentPanel().discardWindow();
            } else {
                getParentPanel().showWindow(new WarningWindow("Unable to update your status", "Unable to update your status. Please try again later.", 30, "notification-sound-1.wav"));
            }
        } catch (APIProtocolException protocolException) {
            _logger.error("", protocolException);
            getParentPanel().showWindow(new WarningWindow("Protocol error", protocolException.getMessage(), 30, "notification-sound-1.wav"));
        } catch (APICommunicationException communicationException) {
            _logger.error("", communicationException);
            getParentPanel().showWindow(new WarningWindow("Communication error", "Unable to update your status due to communication error! Please check your internet connection.", 30, "notification-sound-1.wav"));
        } catch (APIServerErrorException serverErrorException) {
            _logger.error("", serverErrorException);
            getParentPanel().showWindow(new WarningWindow("Server error", "Unable to update your status due to a protocol error! Please contact to admin.", 30, "notification-sound-1.wav"));
        }
    }

    public boolean processVoiceCommand(VoiceCommandVO voiceCommandVO)
    {
        try {
            if (voiceCommandVO.getName() == VoiceCommandAction.GET_TEXT) {
                VoiceCommandVariableVO voiceCommandVariableVO = voiceCommandVO.getVariable("user_text");
                _textPaneMessageText.setText(voiceCommandVariableVO.getValueAsString());
                setTitle("Update Status : Is this correct?");
                textToSpeech("Is this correct?");
                setVoiceCommandState(VoiceCommandState.CONFIRM_MESSAGE);
            } else if (voiceCommandVO.getName() == VoiceCommandAction.CONFIRMATION_YES && getVoiceCommandState() == VoiceCommandState.CONFIRM_MESSAGE) {
                _buttonSend_actionPerformed(null);
            } else if (voiceCommandVO.getName() == VoiceCommandAction.CONFIRMATION_NO && getVoiceCommandState() == VoiceCommandState.CONFIRM_MESSAGE) {
                setTitle("Update Status : Please state your status text...");
                textToSpeech("Please state your status text");
                setVoiceCommandState(VoiceCommandState.GET_MESSAGE_TEXT);
                _textPaneMessageText.setText("");
            } else if (voiceCommandVO.getName() == VoiceCommandAction.SHOW_ATTACHMENT) {
                _buttonOpenAttachment_actionPerformed(null);
            } else {
                return (false);
            }
            return (true);
        } catch (Exception exception) {
            _logger.error("", exception);
        }
        return (false);
    }

    private void textToSpeech(final String text)
    {
        if (Settings.getInstance().getDTO().isTTSCardHeader())
            new Thread(new Runnable()
            {
                public void run()
                {
                    try {
                        TTSPlayer.getInstance().loadPlayfile(API.getInstance().getVoice().tts(text).getAudioURI());
                    } catch (Exception exception) {
                        _logger.error("", exception);
                    }
                }
            }).start();
    }
}
