/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.API;
import net.zypr.api.Protocol;
import net.zypr.api.Session;
import net.zypr.api.enums.ItemDisplay;
import net.zypr.api.enums.MessageType;
import net.zypr.api.vo.ItemVO;
import net.zypr.gui.ImageFetcher;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.ImagePanel;
import net.zypr.gui.components.Label;
import net.zypr.gui.panels.MapViewPanel;
import net.zypr.mmp.mplayer.MPlayer;

import javax.swing.*;

public class RadioStationDetailsWindow
        extends ModalWindow
{
    private ImagePanel _imagePanelRadio = new ImagePanel("placeholder-picture-radio-96.png");
    private Button _buttonSendToContact = new Button("button-station-send-to-contact-up.png", "button-station-send-to-contact-down.png", "button-station-send-to-contact-disabled.png");
    private Button _buttonLocateOnMap = new Button("button-station-locate-on-the-map-up.png", "button-station-locate-on-the-map-down.png", "button-station-locate-on-the-map-disabled.png");
    private Button _buttonPlayStop = new Button("button-station-play-up.png", "button-station-play-down.png");
    private Button _buttonAddRemoveToRoute = new Button("button-station-add-to-route-up.png", "button-station-add-to-route-down.png", "button-station-add-to-route-disabled.png");
    private Button _buttonAddToCalendar = new Button("button-station-add-to-calendar-up.png", "button-station-add-to-calendar-down.png");
    private Button _buttonAddRemoveToFavorites = new Button("button-station-add-to-favorite-up.png", "button-station-add-to-favorite-down.png");
    private Label _labelGenre = new Label();
    private Label _labelDescription = new Label();
    private ItemVO _itemVO = null;
    private boolean _allowSendMessage = true;
    private PropertyChangeListener _mplayerPropertyChangeListener = new PropertyChangeListener()
    {
        public void propertyChange(PropertyChangeEvent propertyChangeEvent)
        {
            if (propertyChangeEvent.getPropertyName().equals("PlaybackStarted"))
                playbackStatusChanged((Boolean) propertyChangeEvent.getNewValue());
        }
    };

    public RadioStationDetailsWindow(ItemVO itemVO, boolean allowSendMessage)
    {
        super();
        try {
            _allowSendMessage = allowSendMessage;
            _itemVO = itemVO;
            jbInit();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        _logger.trace("RadioStationDetailsWindow init");

        this.setSize(new Dimension(560, 240));
        setTitle("Details of \"" + _itemVO.getName() + "\"");

        MigLayout layout = new MigLayout("ins 5, fill");
        _panelContent.setLayout(layout);
        _panelContent.add(_imagePanelRadio, "spany 2");
        _panelContent.add(_labelDescription, "grow, push, wrap");
        _panelContent.add(_labelGenre, "grow, push, wrap");

        _panelContent.add(_buttonAddRemoveToFavorites, "center, spanx, split 6");
        _panelContent.add(_buttonAddToCalendar, "gapbefore 10px");
        _panelContent.add(_buttonAddRemoveToRoute, "gapbefore 10px");
        _panelContent.add(_buttonPlayStop, "gapbefore 10px");
        _panelContent.add(_buttonLocateOnMap, "gapbefore 10px");
        _panelContent.add(_buttonSendToContact, "gapbefore 10px");

        _labelDescription.setText(_itemVO.getDescription());
        _labelDescription.setLineWrap(true);

        _buttonSendToContact.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonSendToContact_actionPerformed(actionEvent);
            }
        });
        _buttonLocateOnMap.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonLocateOnMap_actionPerformed(actionEvent);
            }
        });
        _buttonPlayStop.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonPlayStop_actionPerformed(actionEvent);
            }
        });
        _buttonAddRemoveToRoute.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonAddRemoveToRoute_actionPerformed(actionEvent);
            }
        });
        _buttonAddToCalendar.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonAddToCalendar_actionPerformed(actionEvent);
            }
        });
        _buttonAddRemoveToFavorites.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonAddRemoveToFavorites_actionPerformed(actionEvent);
            }
        });
        if (Session.getInstance().isInFavorites(_itemVO))
            _buttonAddRemoveToFavorites.setIcons("button-station-remove-from-favorite-up.png", "button-station-remove-from-favorite-down.png", "button-station-remove-from-favorite-disabled.png");
        else
            _buttonAddRemoveToFavorites.setIcons("button-station-add-to-favorite-up.png", "button-station-add-to-favorite-down.png", "button-station-add-to-favorite-disabled.png");
        MPlayer.getInstance().addPropertyChangeListener(_mplayerPropertyChangeListener);

        new Thread(new Runnable()
        {
            BufferedImage serviceIcon;
            BufferedImage image = null;

            public void run()
            {
                try {
                    if (!Session.getInstance().getMediaSource().equals(Protocol.SERVICE_NAME_ALL)) {
                        String iconURL = API.getInstance().getMedia().getService(Session.getInstance().getMediaSource()).getServiceIconURI();
                        _logger.debug("getMediaSource icon {}", iconURL);
                        if (iconURL != null && !iconURL.equals("")) {
                            serviceIcon = ImageFetcher.getInstance().getImage(iconURL, 96, 96, true);
                        }
                    }
                    String iconURL = _itemVO.getIconURL();
                    _logger.debug("details icon {}", iconURL);
                    if (iconURL != null && !iconURL.equals("")) {
                        image = ImageFetcher.getInstance().getImage(iconURL, 96, 96, true);
                    }
                } catch (Exception exception) {
                    _logger.error("", exception);
                }

                SwingUtilities.invokeLater(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        if (serviceIcon != null) {
                            setTitleImage(serviceIcon);
                        }
                        if (image != null) {
                            _imagePanelRadio.setOpaque(false);
                            _imagePanelRadio.setImage(image);
                        }
                        revalidate();
                        repaint();
                    }
                });
            }
        }).start();
    }

    private void playbackStatusChanged(boolean started)
    {
        if (isCurrentRadioStation())
            _buttonPlayStop.setIcons("button-media-" + (started ? "stop" : "play") + "-up.png", "button-media-" + (started ? "stop" : "play") + "-down.png", null);
    }

    private boolean isCurrentRadioStation()
    {
        try {
            if (MPlayer.getInstance().getCurrentRadioStationItem() == null)
                return (false);
            return (MPlayer.getInstance().getCurrentRadioStationItem().getGlobalItemID().equals(_itemVO.getGlobalItemID()));
        } catch (NullPointerException nullPointerException) {
            return (false);
        }
    }

    protected void windowVisible()
    {
        playbackStatusChanged(MPlayer.getInstance().isPlaybackStarted());
        _buttonLocateOnMap.setEnabled(_itemVO.getDisplay() == ItemDisplay.GEO);
        _buttonAddRemoveToRoute.setEnabled(_itemVO.getDisplay() == ItemDisplay.GEO);
        _buttonSendToContact.setEnabled(_allowSendMessage);
        MapViewPanel mapViewPanel = (MapViewPanel) getParentPanel();
        mapViewPanel.addItemVOsPOI(_itemVO);
    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent)
    {
        MPlayer.getInstance().removePropertyChangeListener(_mplayerPropertyChangeListener);
        getParentPanel().discardWindow();
    }

    private void _buttonSendToContact_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().showWindow(new SendToContactListWindow(_itemVO, MessageType.TEXT));
    }

    private void _buttonLocateOnMap_actionPerformed(ActionEvent actionEvent)
    {
        ((MapViewPanel) getParentPanel()).setMapCenter(_itemVO.getPosition());
        getParentPanel().discardAllWindows();
    }

    private void _buttonPlayStop_actionPerformed(ActionEvent actionEvent)
    {
        if (isCurrentRadioStation()) {
            try {
                if (MPlayer.getInstance().isPlaybackStarted()) {
                    MPlayer.getInstance().stopPlayback();
                    playbackStatusChanged(false);
                } else {
                    MPlayer.getInstance().loadPlaylist(_itemVO.getAction("play").getHandlerURI());
                    playbackStatusChanged(true);
                }
            } catch (Exception exception) {
                _logger.error("", exception);
            }
        } else {
            MPlayer.getInstance().setCurrentRadioStationItem(_itemVO);
            MPlayer.getInstance().loadPlaylist(_itemVO.getAction("play").getHandlerURI());
        }
    }

    private void _buttonAddRemoveToRoute_actionPerformed(ActionEvent actionEvent)
    {
        _buttonAddRemoveToRoute.setEnabled(false);
        ((MapViewPanel) getParentPanel()).addRoute(_itemVO);
    }

    private void _buttonAddToCalendar_actionPerformed(ActionEvent actionEvent)
    {
        /*- Implement it -*/
        _logger.info(actionEvent.toString());
    }

    private void _buttonAddRemoveToFavorites_actionPerformed(ActionEvent actionEvent)
    {
        if (Session.getInstance().isInFavorites(_itemVO)) {
            try {
                API.getInstance().getUser().favoriteDelete(_itemVO.getGlobalItemID());
                _buttonAddRemoveToFavorites.setIcons("button-station-add-to-favorite-up.png", "button-station-add-to-favorite-down.png", "button-station-add-to-favorite-disabled.png");
            } catch (Exception exception) {
                _logger.error("", exception);
            }
        } else {
            try {
                API.getInstance().getUser().favoriteSet(_itemVO);
                _buttonAddRemoveToFavorites.setIcons("button-station-remove-from-favorite-up.png", "button-station-remove-from-favorite-down.png", "button-station-remove-from-favorite-disabled.png");
            } catch (Exception exception) {
                _logger.error("", exception);
            }
        }
    }
}
