/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.API;
import net.zypr.api.Session;
import net.zypr.api.vo.InfoUserTypeVO;
import net.zypr.gui.Settings;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.CheckBox;
import net.zypr.gui.components.Separator;

import javax.swing.*;

public class SettingsPrivacyWindow extends ModalWindow
{
    private Separator _separator = new Separator(SwingConstants.HORIZONTAL);
    private Button _buttonCancel = new Button("button-voice-note-settings-cancel-up.png", "button-privacy-settings-cancel-down.png");
    private Button _buttonOkay = new Button("button-voice-note-settings-save-up.png", "button-privacy-settings-save-down.png");
    private CheckBox _checkBoxInvisible = new CheckBox("Invisible to Everyone");
    private CheckBox _checkBoxUpdateLocation = new CheckBox("Update My Location");

    public SettingsPrivacyWindow()
    {
        super();
        try {
            jbInit();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        setTitle("Privacy Settings");
        this.setSize(new Dimension(500, 200));

        MigLayout layout = new MigLayout("ins 5, fill", "[210][210]", "");
        _panelContent.setLayout(layout);

        _panelContent.add(_checkBoxInvisible, "");
        _panelContent.add(_checkBoxUpdateLocation, "wrap");
        _panelContent.add(_separator, "grow, gaptop 10px, gapbottom 10px, wrap, spanx");
        _panelContent.add(_buttonOkay, "right, gapright 30px");
        _panelContent.add(_buttonCancel, "gapleft 30px, left, wrap");

        _buttonCancel.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonCancel_actionPerformed(actionEvent);
            }
        });
        _buttonOkay.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonOkay_actionPerformed(actionEvent);
            }
        });
        _checkBoxInvisible.setChecked(!Session.getInstance().getUserInfo().isLocationIsVisible());
        _checkBoxUpdateLocation.setChecked(Settings.getInstance().getDTO().isUpdateMyLocation());
    }

    private void _buttonCancel_actionPerformed(ActionEvent actionEvent)
    {
        _buttonWindowClose_actionPerformed(null);
    }

    private void _buttonOkay_actionPerformed(ActionEvent actionEvent)
    {
        InfoUserTypeVO newInfoUserTypeVO = new InfoUserTypeVO();
        newInfoUserTypeVO.setLocationIsVisible(!_checkBoxInvisible.isChecked());
        try {
            API.getInstance().getUser().infoSet(Session.getInstance().getUserInfo(), newInfoUserTypeVO);
        } catch (Exception exception) {
            getParentPanel().showWindow(new WarningWindow("Unable to save", "Unable to save privacy settings. Please try it later.", 30, "notification-sound-1.wav"));
            return;
        }
        try {
            Session.getInstance().setUserInfo(API.getInstance().getUser().infoGet(null));
        } catch (Exception exception) {
            _logger.error("", exception);
        }
        Settings.getInstance().getDTO().setUpdateMyLocation(_checkBoxUpdateLocation.isChecked());
        Settings.getInstance().save();
        _buttonWindowClose_actionPerformed(null);
    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().discardWindow();
    }
}
