/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.SwingConstants;

import net.miginfocom.swing.MigLayout;
import net.zypr.gui.Settings;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.CheckBox;
import net.zypr.gui.components.Label;
import net.zypr.gui.components.Separator;

public class SettingsTextToSpeechWindow extends ModalWindow
{
    private Separator _separator = new Separator();
    private Button _buttonCancel = new Button("button-text-to-speech-settings-cancel-up.png", "button-text-to-speech-settings-cancel-down.png");
    private Button _buttonOkay = new Button("button-text-to-speech-settings-save-up.png", "button-text-to-speech-settings-save-down.png");
    private Label _labelReadAutomatically = new Label("Read items automatically", SwingConstants.CENTER);
    private CheckBox _checkBoxMessageHeader = new CheckBox("Message Header");
    private CheckBox _checkBoxMessageText = new CheckBox("Message Text");
    private CheckBox _checkBoxContactFeed = new CheckBox("Contact Feed");
    private CheckBox _checkBoxFollowUpList = new CheckBox("Follow Up List");
    private CheckBox _checkBoxCardHeader = new CheckBox("Card Header");

    public SettingsTextToSpeechWindow()
    {
        super();
        try {
            jbInit();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        setTitle("Text To Speech Settings");
        this.setSize(new Dimension(460, 300));

        MigLayout layout = new MigLayout("ins 5, fill");
        _panelContent.setLayout(layout);

        _panelContent.add(_labelReadAutomatically, "grow, wrap, spanx 2");

        _panelContent.add(_checkBoxMessageHeader, "");
        _panelContent.add(_checkBoxMessageText, "wrap");

        _panelContent.add(_checkBoxContactFeed, "");
        _panelContent.add(_checkBoxFollowUpList, "wrap");
        _panelContent.add(_checkBoxCardHeader, "wrap");
        _panelContent.add(_separator, "grow, gaptop 10px, gapbottom 10px, wrap, spanx 2");
        _panelContent.add(_buttonOkay, "right, gapright 40px");
        _panelContent.add(_buttonCancel, "gapleft 40px, left, wrap");

        _buttonCancel.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonCancel_actionPerformed(actionEvent);
            }
        });
        _buttonOkay.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonOkay_actionPerformed(actionEvent);
            }
        });
        _checkBoxMessageHeader.setChecked(Settings.getInstance().getDTO().isTTSMessageHeader());
        _checkBoxMessageText.setChecked(Settings.getInstance().getDTO().isTTSMessageText());
        _checkBoxContactFeed.setChecked(Settings.getInstance().getDTO().isTTSContactFeed());
        _checkBoxFollowUpList.setChecked(Settings.getInstance().getDTO().isTTSFollowUpList());
        _checkBoxCardHeader.setChecked(Settings.getInstance().getDTO().isTTSCardHeader());
    }

    private void _buttonCancel_actionPerformed(ActionEvent actionEvent)
    {
        _buttonWindowClose_actionPerformed(null);
    }

    private void _buttonOkay_actionPerformed(ActionEvent actionEvent)
    {
        Settings.getInstance().getDTO().setTTSMessageHeader(_checkBoxMessageHeader.isChecked());
        Settings.getInstance().getDTO().setTTSMessageText(_checkBoxMessageText.isChecked());
        Settings.getInstance().getDTO().setTTSContactFeed(_checkBoxContactFeed.isChecked());
        Settings.getInstance().getDTO().setTTSFollowUpList(_checkBoxFollowUpList.isChecked());
        Settings.getInstance().getDTO().setTTSCardHeader(_checkBoxCardHeader.isChecked());
        Settings.getInstance().save();
        _buttonWindowClose_actionPerformed(null);
    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().discardWindow();
    }
}
