/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.API;
import net.zypr.api.Session;
import net.zypr.api.vo.InfoUserTypeVO;
import net.zypr.gui.Configuration;
import net.zypr.gui.Resources;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.Label;
import net.zypr.gui.components.PasswordField;
import net.zypr.gui.components.Separator;
import net.zypr.gui.components.TextField;
import net.zypr.gui.utils.StringUtils;

public class SettingsUserWindow extends ModalWindow
{
    private Label _labelEMailAddress = new Label("Email Address");
    private Label _labelPassword = new Label("Password");
    private Label _labelPasswordConfirm = new Label("Password (Confirm)");
    private Label _labelFirstName = new Label("Firstname");
    private Label _labelLastName = new Label("Lastname");
    private TextField _textFieldEMailAddress = new TextField();
    private TextField _textFieldFirstName = new TextField();
    private TextField _textFieldLastName = new TextField();
    private PasswordField _passwordFieldPassword = new PasswordField();
    private PasswordField _passwordFieldPasswordConfirm = new PasswordField();
    private Separator _separatorBottom = new Separator();
    private Button _buttonOkay = new Button("button-user-settings-okay-up.png", "button-user-settings-okay-down.png");
    private Button _buttonCancel = new Button("button-user-settings-cancel-up.png", "button-user-settings-cancel-down.png");

    public SettingsUserWindow()
    {
        super();
        try
        {
            jbInit();
        }
        catch (Exception exception)
        {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        this.setSize(new Dimension(560, 300));
        setTitle("User Settings");

        BufferedImage image = Resources.getInstance().getBufferedImage("logo-service.png");
        setTitleImage(image);

        MigLayout layout = new MigLayout("ins 5, fill", "[][]", "");
        _panelContent.setLayout(layout);
        _panelContent.add(_labelEMailAddress, "grow, wrap, spanx 2");
        _panelContent.add(_textFieldEMailAddress, "grow, spanx 2, wrap");
        _panelContent.add(_labelPassword, "grow");
        _panelContent.add(_labelPasswordConfirm, "grow, wrap");
        _panelContent.add(_passwordFieldPassword, "grow");
        _panelContent.add(_passwordFieldPasswordConfirm, "grow, wrap");
        _panelContent.add(_labelFirstName, "grow");
        _panelContent.add(_labelLastName, "grow, wrap");
        _panelContent.add(_textFieldFirstName, "grow");
        _panelContent.add(_textFieldLastName, "grow, wrap");

        _panelContent.add(_buttonOkay, "center, split 2, spanx");
        _panelContent.add(_buttonCancel, "gapleft 40px");

        _textFieldEMailAddress.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent keyEvent)
            {
                _textFieldEMailAddress_keyPressed(keyEvent);
            }
        });
        _textFieldFirstName.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent keyEvent)
            {
                _textFieldFirstName_keyPressed(keyEvent);
            }
        });
        _textFieldLastName.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent keyEvent)
            {
                _textFieldLastName_keyPressed(keyEvent);
            }
        });
        _passwordFieldPassword.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent keyEvent)
            {
                _passwordFieldPassword_keyPressed(keyEvent);
            }
        });
        _passwordFieldPasswordConfirm.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent keyEvent)
            {
                _passwordFieldPasswordConfirm_keyPressed(keyEvent);
            }
        });
        _buttonOkay.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonOkay_actionPerformed(actionEvent);
            }
        });
        _buttonCancel.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonCancel_actionPerformed(actionEvent);
            }
        });
        _textFieldEMailAddress.setText(Session.getInstance().getUserInfo().getEmail());
        _textFieldFirstName.setText(Session.getInstance().getUserInfo().getFirstName());
        _textFieldLastName.setText(Session.getInstance().getUserInfo().getLastName());
    }

    private void _buttonOkay_actionPerformed(ActionEvent actionEvent)
    {
        _textFieldEMailAddress.setText(_textFieldEMailAddress.getText().trim());
        _textFieldFirstName.setText(_textFieldFirstName.getText().trim());
        _textFieldLastName.setText(_textFieldLastName.getText().trim());
        if (StringUtils.isEmpty(new String(_passwordFieldPassword.getPassword())) && !StringUtils.isEmpty(new String(_passwordFieldPasswordConfirm.getPassword())))
        {
            getParentPanel().showWindow(new WarningWindow("Please enter a password", "Password field is empty. Please enter a password.", 30, "notification-sound-1.wav"));
            return;
        }
        else if (StringUtils.isEmpty(new String(_passwordFieldPasswordConfirm.getPassword())) && !StringUtils.isEmpty(new String(_passwordFieldPassword.getPassword())))
        {
            getParentPanel().showWindow(new WarningWindow("Please enter a password (Confirm)", "Password (Confirm) field is empty. Please enter a password.", 30, "notification-sound-1.wav"));
            return;
        }
        else if (!(new String(_passwordFieldPasswordConfirm.getPassword())).equals((new String(_passwordFieldPassword.getPassword()))))
        {
            getParentPanel().showWindow(new WarningWindow("Passwords mismatch", "Passwords you entered, do not match. Please enter again.", 30, "notification-sound-1.wav"));
            return;
        }
        else if ((new String(_passwordFieldPassword.getPassword())).length() < 4 && (new String(_passwordFieldPassword.getPassword())).length() != 0)
        {
            getParentPanel().showWindow(new WarningWindow("Invalid passwords", "Invalid passwords. Passwords must be at least four characters in length.", 30, "notification-sound-1.wav"));
            return;
        }
        else if (StringUtils.isEmpty(_textFieldFirstName.getText()))
        {
            getParentPanel().showWindow(new WarningWindow("Please enter your first name", "Firstname field is empty. Please enter your first name.", 30, "notification-sound-1.wav"));
            return;
        }
        else if (StringUtils.isEmpty(_textFieldLastName.getText()))
        {
            getParentPanel().showWindow(new WarningWindow("Please enter your last name", "Lastname field is empty. Please enter your first name.", 30, "notification-sound-1.wav"));
            return;
        }
        InfoUserTypeVO newInfoUserTypeVO = new InfoUserTypeVO();
        newInfoUserTypeVO.setFirstName(_textFieldFirstName.getText());
        newInfoUserTypeVO.setLastName(_textFieldLastName.getText());
        newInfoUserTypeVO.setEmail(_textFieldEMailAddress.getText());
        if (!(new String(_passwordFieldPassword.getPassword())).equals(""))
        {
            newInfoUserTypeVO.setCurrentPassword(Session.getInstance().getPassword());
            newInfoUserTypeVO.setNewPassword(new String(_passwordFieldPassword.getPassword()));
            newInfoUserTypeVO.setConfirmNewPassword(new String(_passwordFieldPasswordConfirm.getPassword()));
        }
        try
        {
            API.getInstance().getUser().infoSet(Session.getInstance().getUserInfo(), newInfoUserTypeVO);
        }
        catch (Exception exception)
        {
            getParentPanel().showWindow(new WarningWindow("Error", exception.getMessage(), 30, "notification-sound-1.wav"));
            return;
        }
        try
        {
            Session.getInstance().setUserInfo(API.getInstance().getUser().infoGet(null));
            if (!(new String(_passwordFieldPassword.getPassword())).equals(""))
                Session.getInstance().setPassword((new String(_passwordFieldPassword.getPassword())));
            if (Configuration.getInstance().getBooleanProperty("login-remember-my-password", false))
                Configuration.getInstance().setEncryptedProperty("login-password", Session.getInstance().getPassword());
        }
        catch (Exception exception)
        {
            _logger.error("", exception);
        }
        _buttonWindowClose_actionPerformed(actionEvent);
    }

    private void _buttonCancel_actionPerformed(ActionEvent actionEvent)
    {
        _buttonWindowClose_actionPerformed(actionEvent);
    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().discardWindow();
    }

    private void _textFieldEMailAddress_keyPressed(KeyEvent keyEvent)
    {
        if (keyEvent.getKeyCode() == KeyEvent.VK_ENTER)
            _buttonOkay_actionPerformed(null);
    }

    private void _textFieldFirstName_keyPressed(KeyEvent keyEvent)
    {
        if (keyEvent.getKeyCode() == KeyEvent.VK_ENTER)
            _buttonOkay_actionPerformed(null);
    }

    private void _textFieldLastName_keyPressed(KeyEvent keyEvent)
    {
        if (keyEvent.getKeyCode() == KeyEvent.VK_ENTER)
            _buttonOkay_actionPerformed(null);
    }

    private void _passwordFieldPassword_keyPressed(KeyEvent keyEvent)
    {
        if (keyEvent.getKeyCode() == KeyEvent.VK_ENTER)
            _buttonOkay_actionPerformed(null);
    }

    private void _passwordFieldPasswordConfirm_keyPressed(KeyEvent keyEvent)
    {
        if (keyEvent.getKeyCode() == KeyEvent.VK_ENTER)
            _buttonOkay_actionPerformed(null);
    }

    protected void _this_componentShown(ComponentEvent componentEvent)
    {
        _textFieldEMailAddress.requestFocus();
    }
}
