/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.SwingConstants;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.API;
import net.zypr.api.Session;
import net.zypr.gps.GPSDevice;
import net.zypr.gui.Configuration;
import net.zypr.gui.Settings;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.Label;
import net.zypr.gui.components.Separator;
import net.zypr.gui.panels.MapViewPanel;
import net.zypr.mmp.mplayer.MPlayer;

public class SettingsWindow extends ModalWindow
{
    private Button _buttonVoiceNote = new Button("button-settings-voice-note-up.png", "button-settings-voice-note-down.png");
    private Button _buttonTextToSpeech = new Button("button-settings-text-to-speech-up.png", "button-settings-text-to-speech-down.png");
    private Button _buttonMediaSource = new Button("button-settings-media-source-up.png", "button-settings-media-source-down.png");
    private Button _buttonPrivacy = new Button("button-settings-privacy-up.png", "button-settings-privacy-down.png");
    private Button _buttonServices = new Button("button-settings-services-up.png", "button-settings-services-down.png");
    private Button _buttonUnits = new Button("button-settings-units-up.png", "button-settings-units-down.png");
    private Button _buttonUser = new Button("button-settings-user-up.png", "button-settings-user-down.png");
    private Button _buttonSignOut = new Button("button-sign-out-up.png", "button-sign-out-down.png");
    private Button _buttonExit = new Button("button-settings-exit-up.png", "button-settings-exit-down.png");
    private Label _labelVoiceNote = new Label("Voice Note", SwingConstants.CENTER);
    private Label _labelTextToSpeech = new Label("Text To Speech", SwingConstants.CENTER);
    private Label _labelMediaSource = new Label("Media Source", SwingConstants.CENTER);
    private Label _labelPrivacy = new Label("Privacy", SwingConstants.CENTER);
    private Label _labelServices = new Label("Services", SwingConstants.CENTER);
    private Label _labelUnits = new Label("Units", SwingConstants.CENTER);
    private Label _labelUser = new Label("User", SwingConstants.CENTER);
    private Label _labelSignOut = new Label("Sign Out", SwingConstants.CENTER);
    private Label _labelExit = new Label("Exit", SwingConstants.CENTER);

    public SettingsWindow()
    {
        super();
        try {
            jbInit();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        _logger.trace("SettingsWindow init");

        this.setSize(new Dimension(530, 300));
        this.setTitle("Settings");

        MigLayout layout = new MigLayout("ins 5, center, fill", "[150][150][150]", "");
        _panelContent.setLayout(layout);

        _panelContent.add(_buttonServices, "center");
        _panelContent.add(_buttonUser, "center, wrap");
        _panelContent.add(_labelServices, "grow");
        _panelContent.add(_labelUser, "grow, wrap");

        _panelContent.add(_buttonVoiceNote, "center");
        _panelContent.add(_buttonTextToSpeech, "center");
        _panelContent.add(_buttonUnits, "center, wrap");
        _panelContent.add(_labelVoiceNote, "grow");
        _panelContent.add(_labelTextToSpeech, "grow");
        _panelContent.add(_labelUnits, "grow, wrap");

        _panelContent.add(_buttonPrivacy, "center");
        _panelContent.add(_buttonSignOut, "center");
        _panelContent.add(_buttonExit, "center, wrap");
        _panelContent.add(_labelPrivacy, "grow");
        _panelContent.add(_labelSignOut, "grow");
        _panelContent.add(_labelExit, "grow");

        // TODO: why?
        _buttonMediaSource.setVisible(false);
        _labelMediaSource.setVisible(false);

        // TODO: consolidate
        _buttonVoiceNote.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonVoiceNote_actionPerformed(actionEvent);
            }
        });
        _buttonTextToSpeech.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonTextToSpeech_actionPerformed(actionEvent);
            }
        });
        _buttonMediaSource.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonMediaSource_actionPerformed(actionEvent);
            }
        });
        _buttonPrivacy.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonPrivacy_actionPerformed(actionEvent);
            }
        });
        _buttonServices.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonServices_actionPerformed(actionEvent);
            }
        });
        _buttonUnits.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonUnits_actionPerformed(actionEvent);
            }
        });
        _buttonUser.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonUser_actionPerformed(actionEvent);
            }
        });
        _buttonSignOut.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonSignOut_actionPerformed(actionEvent);
            }
        });
        _buttonExit.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                _buttonExit_actionPerformed(actionEvent);
            }
        });
    }

    protected void windowInvisible()
    {
        Session.getInstance().forceUpdateBGProcesses();
    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().discardWindow();
    }

    private void _buttonVoiceNote_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().showWindow(new SettingsVoiceNoteWindow());
    }

    private void _buttonTextToSpeech_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().showWindow(new SettingsTextToSpeechWindow());
    }

    private void _buttonMediaSource_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().showWindow(new SettingsMediaSourceWindow());
    }

    private void _buttonPrivacy_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().showWindow(new SettingsPrivacyWindow());
    }

    private void _buttonServices_actionPerformed(ActionEvent actionEvent)
    {
        _buttonServices.setEnabled(false);
        new Thread(new Runnable()
        {
            public void run()
            {
                try {
                    getParentPanel().showWindow(new SettingsServiceWindow(API.getInstance().getAuth().serviceAuthStatus()));
                } catch (Exception exception) {
                    _buttonServices.setEnabled(true);
                    _logger.error("", exception);
                }
            }
        }).start();
    }

    private void _buttonUnits_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().showWindow(new SettingsUnitsWindow());
    }

    private void _buttonUser_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().showWindow(new SettingsUserWindow());
    }

    private void _buttonSignOut_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().discardAllWindows();
        Configuration.getInstance().save();
        Settings.getInstance().save();
        Settings.getInstance().clear();
        if (MPlayer.getInstance().isPlaybackStarted())
            MPlayer.getInstance().stopPlayback();
        MPlayer.getInstance().removeAllPropertyChangeListener();
        GPSDevice.getInstance().removeAllPropertyChangeListener();
        try {
            API.getInstance().getAuth().logout();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
        Session.getInstance().loggedOut();
        Configuration.getInstance().setBooleanProperty("login-sign-me-in-automatically", false);
        getParentPanel().getMainPanel().showStartupPanel();
    }

    private void _buttonExit_actionPerformed(ActionEvent actionEvent)
    {
        ((MapViewPanel) getParent()).saveAsImage();
        Configuration.getInstance().save();
        Settings.getInstance().save();
        try {
            API.getInstance().getAuth().logout();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
        System.exit(0);
    }
}
