/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.API;
import net.zypr.api.Session;
import net.zypr.api.enums.StatusCode;
import net.zypr.api.exceptions.APICommunicationException;
import net.zypr.api.exceptions.APIProtocolException;
import net.zypr.api.vo.InfoUserTypeVO;
import net.zypr.gui.Branding;
import net.zypr.gui.Resources;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.Label;
import net.zypr.gui.components.PasswordField;
import net.zypr.gui.components.Separator;
import net.zypr.gui.components.TextField;
import net.zypr.gui.utils.DeviceID;
import net.zypr.gui.utils.StringUtils;

public class SignUpWindow extends ModalWindow implements KeyListener, ActionListener
{
    private Label _labelEMailAddress = new Label("Email Address");
    private Label _labelPassword = new Label("Password");
    private Label _labelPasswordConfirm = new Label("Password (Confirm)");
    private Label _labelFirstName = new Label("Firstname");
    private Label _labelLastName = new Label("Lastname");
    private TextField _textFieldEMailAddress = new TextField();
    private TextField _textFieldFirstName = new TextField();
    private TextField _textFieldLastName = new TextField();
    private PasswordField _passwordFieldPassword = new PasswordField();
    private PasswordField _passwordFieldPasswordConfirm = new PasswordField();
    private Separator _separatorTop = new Separator();
    private Separator _separatorBottom = new Separator();
    private Button _buttonOkay = new Button("button-sign-up-okay-up.png", "button-sign-up-okay-down.png");
    private Button _buttonCancel = new Button("button-sign-up-cancel-up.png", "button-sign-up-cancel-down.png");

    static final private int SIGN_UP_FONT_SIZE = 20;

    public SignUpWindow()
    {
        super();
        try {
            jbInit();
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        this.setSize(new Dimension(620, 340));
        String title = Branding.getInstance().getStringProperty("signup-title-text", "Sign Up");
        setTitle(title);

        BufferedImage image = Resources.getInstance().getBufferedImage("logo-service.png");
        setTitleImage(image);

        MigLayout layout = new MigLayout("ins 5, fill", "[][]", "");
        _panelContent.setLayout(layout);
        _panelContent.add(_separatorTop, "gaptop 5px, gapbottom 5px, grow, wrap, spanx");
        _panelContent.add(_labelEMailAddress, "grow, wrap, spanx 2");
        _panelContent.add(_textFieldEMailAddress, "grow, spanx 2, wrap");
        _panelContent.add(_labelPassword, "grow");
        _panelContent.add(_labelPasswordConfirm, "grow, wrap");
        _panelContent.add(_passwordFieldPassword, "grow");
        _panelContent.add(_passwordFieldPasswordConfirm, "grow, wrap");
        _panelContent.add(_labelFirstName, "grow");
        _panelContent.add(_labelLastName, "grow, wrap");
        _panelContent.add(_textFieldFirstName, "grow");
        _panelContent.add(_textFieldLastName, "grow, wrap");
        _panelContent.add(_separatorBottom, "gaptop 5px, gapbottom 10px, grow, spanx");

        _panelContent.add(_buttonOkay, "center, split 2, spanx");
        _panelContent.add(_buttonCancel, "gapleft 40px");

        Font labelFont = new Font("Dialog", Font.PLAIN, 14);
        _labelEMailAddress.setFont(labelFont);
        _labelPassword.setFont(labelFont);
        _labelPasswordConfirm.setFont(labelFont);
        _labelFirstName.setFont(labelFont);
        _labelLastName.setFont(labelFont);

        Font textFont = new Font("Dialog", Font.PLAIN, SIGN_UP_FONT_SIZE);
        _textFieldEMailAddress.setFont(textFont);
        _textFieldEMailAddress.addKeyListener(this);
        _textFieldFirstName.setFont(textFont);
        _textFieldFirstName.addKeyListener(this);
        _textFieldLastName.setFont(textFont);
        _textFieldLastName.addKeyListener(this);
        _passwordFieldPassword.setFont(textFont);
        _passwordFieldPassword.addKeyListener(this);
        _passwordFieldPasswordConfirm.setFont(textFont);
        _passwordFieldPasswordConfirm.addKeyListener(this);
        _buttonOkay.addActionListener(this);
        _buttonCancel.addActionListener(this);
    }

    private void processSignUp()
    {
        _textFieldEMailAddress.setText(_textFieldEMailAddress.getText().trim());
        _textFieldFirstName.setText(_textFieldFirstName.getText().trim());
        _textFieldLastName.setText(_textFieldLastName.getText().trim());
        if (StringUtils.isEmpty(new String(_passwordFieldPassword.getPassword()))) {
            getParentPanel().showWindow(new WarningWindow("Please enter a password", "Password field is empty. Please enter a password.", 30, "notification-sound-1.wav"));
            return;
        } else if (StringUtils.isEmpty(new String(_passwordFieldPasswordConfirm.getPassword()))) {
            getParentPanel().showWindow(new WarningWindow("Please enter a password (Confirm)", "Password (Confirm) field is empty. Please enter a password.", 30, "notification-sound-1.wav"));
            return;
        } else if (!(new String(_passwordFieldPasswordConfirm.getPassword())).equals((new String(_passwordFieldPassword.getPassword())))) {
            getParentPanel().showWindow(new WarningWindow("Passwords mismatch", "Passwords you entered, do not match. Please enter again.", 30, "notification-sound-1.wav"));
            return;
        } else if ((new String(_passwordFieldPassword.getPassword())).length() < 4) {
            getParentPanel().showWindow(new WarningWindow("Invalid passwords", "Invalid passwords. Passwords must be at least four characters in length.", 30, "notification-sound-1.wav"));
            return;
        } else if (StringUtils.isEmpty(_textFieldFirstName.getText())) {
            getParentPanel().showWindow(new WarningWindow("Please enter your first name", "Firstname field is empty. Please enter your first name.", 30, "notification-sound-1.wav"));
            return;
        } else if (StringUtils.isEmpty(_textFieldLastName.getText())) {
            getParentPanel().showWindow(new WarningWindow("Please enter your last name", "Lastname field is empty. Please enter your first name.", 30, "notification-sound-1.wav"));
            return;
        }
        try {
            if (API.getInstance().getAuth().createUser(_textFieldEMailAddress.getText(), new String(_passwordFieldPassword.getPassword()), new String(_passwordFieldPasswordConfirm.getPassword())) == StatusCode.SUCCESSFUL) {
                if (API.getInstance().getAuth().login(_textFieldEMailAddress.getText(), new String(_passwordFieldPasswordConfirm.getPassword()), DeviceID.load(_textFieldEMailAddress.getText())) == StatusCode.SUCCESSFUL) {
                    DeviceID.save(_textFieldEMailAddress.getText(), Session.getInstance().getDeviceID());
                    InfoUserTypeVO _infoUserTypeVO = new InfoUserTypeVO();
                    _infoUserTypeVO.setFirstName(_textFieldFirstName.getText());
                    _infoUserTypeVO.setLastName(_textFieldLastName.getText());
                    try {
                        API.getInstance().getUser().infoSet(Session.getInstance().getUserInfo(), _infoUserTypeVO);
                    } catch (Exception exception) {
                        _logger.error("", exception);
                    }
                    try {
                        Session.getInstance().setUserInfo(API.getInstance().getUser().infoGet(null));
                    } catch (Exception exception) {
                        _logger.error("", exception);
                    }
                    getParentPanel().getMainPanel().showMapViewPanel();
                }
            } else {
                getParentPanel().showWindow(new WarningWindow("Unable to sign up", "Unable to sign up. Please try another e-mail address.", 30, "notification-sound-1.wav"));
                return;
            }
        } catch (APIProtocolException protocolException) {
            _logger.error("", protocolException);
            getParentPanel().showWindow(new WarningWindow("Protocol error", protocolException.getMessage(), 30, "notification-sound-1.wav"));
            return;
        } catch (APICommunicationException communicationException) {
            _logger.error("", communicationException);
            getParentPanel().showWindow(new WarningWindow("Communication error", "Unable to sign up due to communication error! Please check your internet connection.", 30, "notification-sound-1.wav"));
            return;
        } catch (Exception exception) {
            _logger.error("", exception);
            getParentPanel().showWindow(new WarningWindow("Error", exception.getMessage(), 30, "notification-sound-1.wav"));
            return;
        }
        // Close the window
        getParentPanel().discardWindow();
    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent)
    {
        getParentPanel().discardWindow();
    }

    protected void _this_componentShown(ComponentEvent componentEvent)
    {
        _textFieldEMailAddress.requestFocus();
    }

    @Override
    public void keyTyped(KeyEvent keyEvent)
    {
        if (keyEvent.getKeyCode() == KeyEvent.VK_ENTER) {
            processSignUp();
        }
    }

    @Override
    public void keyPressed(KeyEvent keyEvent)
    {
    }

    @Override
    public void keyReleased(KeyEvent keyEvent)
    {
    }

    @Override
    public void actionPerformed(ActionEvent actionEvent)
    {
        if (actionEvent.getSource() == _buttonOkay) {
            processSignUp();
        } else if (actionEvent.getSource() == _buttonCancel) {
            // Close the window
            getParentPanel().discardWindow();
        }
    }
}
