/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import net.miginfocom.swing.MigLayout;
import net.zypr.gui.Resources;
import net.zypr.gui.components.Button;
import net.zypr.gui.components.ImagePanel;
import net.zypr.gui.components.Label;
import net.zypr.gui.components.Separator;
import net.zypr.gui.utils.AudioUtils;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;

public class ToastWindow extends ModalWindow
{
    private ImagePanel _imagePanelIcon;
    private Label _labelMessage = new Label("<html>...</html>");
    private Timer _timerClose = null;

    public ToastWindow(String title, String message, int seconds, String audioName, String imageName)
    {
        super();
        try {
            if (seconds > 0)
                _timerClose = new Timer(seconds * 1000, new ActionListener()
                {
                    public void actionPerformed(ActionEvent actionEvent)
                    {
                        _buttonWindowClose_actionPerformed(actionEvent);
                    }
                });
            setTitle(title);
            _labelMessage.setText("<html>" + message + "</html>");
            _imagePanelIcon = new ImagePanel(imageName, false);
            jbInit();
            if (audioName != null)
                AudioUtils.play(audioName);
        } catch (Exception exception) {
            _logger.error("", exception);
        }
    }

    private void jbInit()
            throws Exception
    {
        this.setSize(400, 280);

        BufferedImage image = Resources.getInstance().getBufferedImage("logo-service.png");
        setTitleImage(image);

        MigLayout layout = new MigLayout("ins 5, fill");
        _panelContent.setLayout(layout);
        _panelContent.add(_labelMessage, "center, grow, wrap");
        _panelContent.add(_imagePanelIcon, "center, width 340!, height 180!, wrap");

        disableButtonClose();

        if (_timerClose != null)
            _timerClose.start();
    }
}
