/**
 *
 * Initial version of this code (c) 2009-2011 Media Tuners LLC with a full license to Pioneer Corporation.
 *
 * Pioneer Corporation licenses this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 *
 */

package net.zypr.gui.windows;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;

import java.awt.event.MouseListener;
import java.text.SimpleDateFormat;

import javax.swing.*;

import net.miginfocom.swing.MigLayout;
import net.zypr.api.API;
import net.zypr.api.Weather;
import net.zypr.api.enums.CardinalDirection;
import net.zypr.api.enums.TemperatureScale;
import net.zypr.api.vo.WeatherCurrentVO;
import net.zypr.gui.ImageFetcher;
import net.zypr.gui.Settings;
import net.zypr.gui.components.ImagePanel;
import net.zypr.gui.components.Label;
import net.zypr.gui.utils.Converter;
import net.zypr.gui.utils.WebBrowser;

public class WeatherCurrentWindow extends ModalWindow implements MouseListener {
    private WeatherCurrentVO _weatherCurrentVO = null;
    private String _placename = null;
    private String _service = null;
    private ImagePanel weatherImage = new ImagePanel();
    private Label timeLabel = new Label();
    private Label descriptionLabel = new Label();
    private Label tempLabel = new Label();
    private Label windLabel1 = new Label();
    private Label windLabel2 = new Label();

    public WeatherCurrentWindow(String service, WeatherCurrentVO weatherCurrentVO, String placename) {
        try {
            _service = service;
            _weatherCurrentVO = weatherCurrentVO;
            _placename = placename;
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        this.setSize(new Dimension(350, 250));
        setTitle(_placename);

        // TODO: run this in a separate thread
        if (_service == null || _service.equalsIgnoreCase(Weather.SERVICE_NAME_DEFAULT))
            _service = API.getInstance().getWeather().getDefaultServiceForCurrent();
        setTitleImage(ImageFetcher.getInstance().getImage(API.getInstance().getWeather().getService(_service).getServiceIconURI()));

        MigLayout layout = new MigLayout("insets 5","[160]","[fill]0[fill]");
        _panelContent.setLayout(layout);
        _panelContent.add(weatherImage, "width 160!, height 160!, spany");
        _panelContent.add(timeLabel, "width 160!, wrap");
        _panelContent.add(descriptionLabel, "width 160!, wrap, gaptop 10px, gapbottom 10px");
        _panelContent.add(tempLabel, "width 160!, height 80!, wrap");
        _panelContent.add(windLabel1, "width 160!, wrap, gaptop 5px");
        _panelContent.add(windLabel2, "width 160!");

        weatherImage.setBackground(new Color(0, 0, 0, 0));
        //weatherImage.setImage(ImageFetcher.getInstance().getImage(_weatherCurrentVO.getIconURI(), 120, 120, true));
        weatherImage.setImage(ImageFetcher.getInstance().getImage(_weatherCurrentVO.getIconURI(), 0, 0, true));

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("h:mm a, z");
        timeLabel.setText(simpleDateFormat.format(_weatherCurrentVO.getDate()));
        timeLabel.setHorizontalAlignment(SwingConstants.CENTER);
        timeLabel.setFont(new Font("Dialog", Font.PLAIN, 12));

        //descriptionLabel.setText("<html>" + _weatherCurrentVO.getDescriptionLong() + "</html>");
        descriptionLabel.setText(_weatherCurrentVO.getDescriptionLong());
        descriptionLabel.setHorizontalAlignment(SwingConstants.CENTER);
        descriptionLabel.setFont(new Font("Dialog",Font.PLAIN,16));

        TemperatureScale tempDisplayScale = Settings.getInstance().getDTO().getTemperatureScale();
        long temp = (long) Converter.convertTemperatures(_weatherCurrentVO.getScale(), tempDisplayScale, _weatherCurrentVO.getCurrentTemperature());

        tempLabel.setText(temp + tempDisplayScale.getSymbol());
        tempLabel.setHorizontalAlignment(SwingConstants.CENTER);
        tempLabel.setFont(new Font("Dialog",Font.PLAIN,36));

        windLabel1.setText("Wind");
        windLabel1.setHorizontalAlignment(SwingConstants.CENTER);
        windLabel1.setFont(new Font("Dialog",Font.PLAIN,12));

        long wind = _weatherCurrentVO.getWindSpeed();
        long gust = _weatherCurrentVO.getWindGust();
        CardinalDirection dir = _weatherCurrentVO.getWindDirection();
        // TODO: support mph and kmh
        if (wind == gust) {
            windLabel2.setText(dir.toString() + " at " + wind + "mph");
        } else {
            windLabel2.setText(dir.toString() + " at " + wind + " to " + gust + " mph");
        }
        windLabel2.setHorizontalAlignment(SwingConstants.CENTER);
        windLabel2.setFont(new Font("Dialog",Font.PLAIN,12));

        this.addMouseListener(this);
    }

    protected void _buttonWindowClose_actionPerformed(ActionEvent actionEvent) {
        getParentPanel().discardWindow();
    }

    @Override
    public void mouseClicked(MouseEvent mouseEvent) {
        WebBrowser.openURL(_weatherCurrentVO.getDetailsURI());
    }

    @Override
    public void mousePressed(MouseEvent mouseEvent) {
    }

    @Override
    public void mouseReleased(MouseEvent mouseEvent) {
    }

    @Override
    public void mouseEntered(MouseEvent mouseEvent) {
    }

    @Override
    public void mouseExited(MouseEvent mouseEvent) {
    }
}
